// Mandelbrot.c
//
// Using integer math, generating ASCII output
//
// Wayne Parham

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifndef MIPS
#define MIPS            -1
#endif
#define DISPLAY_COLS    80
#define DISPLAY_ROWS    23
#define INIT_LEFT     -560  // (x size 800, 80 steps)
#define INIT_RIGHT     240
#define INIT_TOP       220  // (y size 460, 23 steps)
#define INIT_BOTTOM   -240
#define INIT_X_INCR     10  // (step ratio 2:1, y:x )
#define INIT_Y_INCR     20
#define INIT_MAX_ITER   50


void mandelbrot( long boundL, long boundR, long boundT, long boundB,
                 long x_incr,long y_incr, long max, char* disbuf, char disp ) {

   long n,cx,cy,zx,zy,zxx,zyy;
   char p;
   int  i;

   i = 0;

   for( cy=boundT; cy > boundB; cy -= y_incr ) {
      for( cx=boundL; cx < boundR; cx += x_incr ) {
         n  = 0;
         zy = 0;
         zx = 0;
         p  = ' ';
	 while( n < max ) {
            zxx = zx * zx / 200;
            zyy = zy * zy / 200;
            if( (zxx + zyy) > 800 ) {
               if( n > 9 ) 
                  p = '0';
               else
                  p = n + 48;
               n = max;
            }
            zy = zx * zy / 100 + cy;
            zx = zxx - zyy + cx;
            n++;
         }
         disbuf[i] = p;
	 i++;
         if( disp )
            putchar( p );
      }
      if( disp )
         putchar( '\n' );
   }
   return;
}


void display( char* disbuf ) {
   int i,x,y;
   for( y=0; y < DISPLAY_ROWS; y++ ) {
      putchar( '\n' );
      for( x=0; x < DISPLAY_COLS; x++ ) {
         putchar( disbuf[i] );
         i++;
      }
   }
   return;
}


int getnum() {
   char  input[DISPLAY_COLS];
   char* check;
   int   value;
   if( !fgets(input, sizeof input, stdin) ) {
      value = -1;
   } else {
      if( ! strchr(input, '\n') ) {
         while( fgets(input, sizeof input, stdin) && !strchr(input, '\n') );
      } else {
         value = (int) strtol(input, &check, 10);
         if( ! ( (isspace(*check)) || (*check == 0) ) ) {
            value = -1;
         }
      }
   }

   return( value );
}


#ifdef USE_GETCHAR
char getcmd() {
   char cmd;
   cmd = getchar();
   putchar( 0x08 );
   switch( cmd ) {
      case 'A' :
      case 'a' :
         puts( "Entering auto mode." );
         break;
      case 'L' :
      case 'l' :
         puts( "Moving left and re-rendering." );
         break;
      case 'R' :
      case 'r' :
         puts( "Moving right and re-rendering." );
         break;
      case 'U' :
      case 'u' :
         puts( "Moving up and re-rendering." );
         break;
      case 'D' :
      case 'd' :
         puts( "Moving down and re-rendering." );
         break;
      case '.' :
         puts( "Re-centering and re-rendering." );
         break;
      case '+' :
         puts( "Zooming in and re-rendering." );
         break;
      case '-' :
         puts( "Zooming out and re-rendering." );
         break;
      case 'X' :
      case 'x' :
         puts( "Enjoy your day!" );
         break;
      default  :
         break;
   }
   return( cmd );
}
#else


char getcmd() {
   char  input[DISPLAY_COLS];
   char  cmd;
   if( !fgets(input, sizeof input, stdin) ) {
      cmd = 0x00;
   } else {
      if( ! strchr(input, '\n') ) {
         while( fgets(input, sizeof input, stdin) && !strchr(input, '\n') );
      } else {
         cmd = input[0];
      }
   }

   return( cmd );
}
#endif


void delay( long count ) {
   long j;
   for( j=0; j<count; j++ ) {
   }
}


void autorender( long boundL, long boundR, long boundT, long boundB,
                 long x_incr, long y_incr, long max, char* disbuf, long hold_time ) {

   long i,plot_points,setnum,setratio,move;
   plot_points = DISPLAY_COLS * DISPLAY_ROWS;

   while( 1 ) {
      setnum = 0;
      for( i=0; i < plot_points; i++ ) {
         if( disbuf[i] == ' ' ) {
            setnum++;
         }
      }
      setratio = 100 * setnum / plot_points;
      if( (setratio > 90) || (setratio < 10) ) {
         boundL = INIT_LEFT;
         boundR = INIT_RIGHT;
         boundT = INIT_TOP;
         boundB = INIT_BOTTOM;
         x_incr = INIT_X_INCR;
         y_incr = INIT_Y_INCR;
         max    = INIT_MAX_ITER;
      } else {
         move = rand() % 6;
         switch( move ) {
            case 0 :
               if( x_incr > 1 ) {
                  if( x_incr != 5 ) {
                     x_incr = x_incr / 2;
                     y_incr = y_incr / 2;
                     boundL = boundL / 2;
                     boundR = boundR / 2;
                     boundT = boundT / 2;
                     boundB = boundB / 2;
                  } else {
                     x_incr = x_incr * 10 / 25;
                     y_incr = y_incr * 10 / 25;
                     boundL = boundL * 10 / 25;
                     boundR = boundR * 10 / 25;
                     boundT = boundT * 10 / 25;
                     boundB = boundB * 10 / 25;
                  }
               }
               break;
            case 1 :
               if( x_incr < 1280 ) {
                  if( x_incr != 2 ) {
                     x_incr = x_incr * 2;
                     y_incr = y_incr * 2;
                     boundL = boundL * 2;
                     boundR = boundR * 2;
                     boundT = boundT * 2;
                     boundB = boundB * 2;
                  } else {
                     x_incr = x_incr * 25 / 10;
                     y_incr = y_incr * 25 / 10;
                     boundL = boundL * 25 / 10;
                     boundR = boundR * 25 / 10;
                     boundT = boundT * 25 / 10;
                     boundB = boundB * 25 / 10;
                  }
               }
               break;
            case 2 :
               boundL = boundL - x_incr;
               boundR = boundR - x_incr;
               break;
            case 3 :
               boundL = boundL + x_incr;
               boundR = boundR + x_incr;
               break;
            case 4 :
               boundT = boundT + y_incr;
               boundB = boundB + y_incr;
               break;
            case 5 :
               boundT = boundT - y_incr;
               boundB = boundB - y_incr;
               break;
         }
      }

      mandelbrot( boundL, boundR, boundT, boundB, x_incr, y_incr, max, disbuf, 0 );
      delay( hold_time );
      display( disbuf );
   }
}


void instructions( void ) {
   puts( "\r" );
   puts( "The Mandelbrot set includes all two-dimensional c values for which the function" );
   puts( "z = z^2 + c does not diverge when iterated from z = 0.  It was named in tribute" );
   puts( "to the mathematician Benoit Mandelbrot, a pioneer of fractal geometry.   He was" );
   puts( "first  to see a visualization of the set on March 1, 1980, at IBM's  Thomas  J." );
   puts( "Watson Research Center in Yorktown Heights, New York." );
   puts( "\r" );
   puts( "The  Mandelbrot  set  has  its  origin  in  complex  dynamics,  a  field  first" );
   puts( "investigated by the French mathematicians Pierre Fatou and Gaston Julia at  the" );
   puts( "beginning of the 20th century. This fractal was first defined and drawn in 1978" );
   puts( "by Robert W. Brooks and Peter Matelski as part of a study of Kleinian groups." );
   puts( "\r" );
   puts( "Mandelbrot  studied the parameter space of quadratic polynomials in an  article" );
   puts( "that  appeared  in 1980.  The mathematical study of the Mandelbrot  set  really" );
   puts( "began with work by the mathematicians Adrien Douady and John H. Hubbard (1985)," );
   puts( "who  established many of its fundamental properties and named the set in  honor" );
   puts( "of Mandelbrot for his influential work in fractal geometry." );
   puts( "\r" );
   puts( "This  program  displays the set on an ASCII terminal, much  like  the  earliest" );
   puts( "computers did.  It is a throwback to the computing technologies of  yesteryear." );
   puts( "It will display the set and then allow you to zoom or scroll. You can zoom with" );
   puts( "'+'  or '-' or move in any direction with 'u', 'd', 'l' or 'r'.   Enter '.'  to" );
   puts( "reset the view.  Set auto mode with the 'a' key.   Press ENTER to continue." );

   return;
}


void main( void ) {
   int   mips;
   int   changed;
   int   running;
   long  hold_time;
   long  boundL = INIT_LEFT;
   long  boundR = INIT_RIGHT;
   long  boundT = INIT_TOP;
   long  boundB = INIT_BOTTOM;
   long  x_incr = INIT_X_INCR;
   long  y_incr = INIT_Y_INCR;
   long  max    = INIT_MAX_ITER;
   char* disbuf;
   char  cmd;

   disbuf = (char *) malloc (DISPLAY_COLS * (DISPLAY_ROWS+1));
   if( disbuf == 0x00 ) {
      puts( "Memory full.  Cannot allocate memory so program is aborting." );
   } else {
      memset( disbuf, 0x00, DISPLAY_COLS * (DISPLAY_ROWS+1) );
      if( MIPS >= 0 ) {
         mips = MIPS;     // Compiling with -DMIPS=<cpu_mips> will omit the cpu speed question
      } else {
         printf( "\nProcessor speed (in MIPS)? " );
         mips = getnum();
      }
      if( mips >= 1 )
         hold_time = 1000000 * mips;  // one second display for each rendering
      else
         hold_time = 0;               // low-speed processors don't need delay

      srand( (unsigned long)&autorender );

      instructions();
      cmd = getcmd();
      
      mandelbrot( boundL, boundR, boundT, boundB, x_incr, y_incr, max, disbuf, 1 );
      
      if( cmd == 'a' ) {
         autorender( boundL, boundR, boundT, boundB, x_incr, y_incr, max, disbuf, hold_time );
      } else {
         running = 1;
         while( running ) {
            cmd = getcmd();
            changed = 1;
            switch( cmd ) {
               case 'A' :
               case 'a' :
                  autorender( boundL, boundR, boundT, boundB, x_incr, y_incr, max, disbuf, hold_time );
                  break;
               case 'X' :
               case 'x' :
                  running = 0;
                  break;
               case 'L' :
               case 'l' :
                  boundL = boundL - x_incr;
                  boundR = boundR - x_incr;
                  break;
               case 'R' :
               case 'r' :
                  boundL = boundL + x_incr;
                  boundR = boundR + x_incr;
                  break;
               case 'U' :
               case 'u' :
                  boundT = boundT + y_incr;
                  boundB = boundB + y_incr;
                  break;
               case 'D' :
               case 'd' :
                  boundT = boundT - y_incr;
                  boundB = boundB - y_incr;
                  break;
               case '.' :
                  boundL = INIT_LEFT;
                  boundR = INIT_RIGHT;
                  boundT = INIT_TOP;
                  boundB = INIT_BOTTOM;
                  x_incr = INIT_X_INCR;
                  y_incr = INIT_Y_INCR;
                  break;
               case '+' :
                  if( x_incr > 1 ) {
                     if( x_incr != 5 ) {
                        x_incr = x_incr / 2;
                        y_incr = y_incr / 2;
                        boundL = boundL / 2;
                        boundR = boundR / 2;
                        boundT = boundT / 2;
                        boundB = boundB / 2;
                     } else {
                        x_incr = x_incr * 10 / 25;
                        y_incr = y_incr * 10 / 25;
                        boundL = boundL * 10 / 25;
                        boundR = boundR * 10 / 25;
                        boundT = boundT * 10 / 25;
                        boundB = boundB * 10 / 25;
                     }
                  } else {
                     printf ( "\nMaximum zoom reached." );
                  }
                  break;
               case '-' :
                  if( x_incr < 1280 ) {
                     if( x_incr != 2 ) {
                        x_incr = x_incr * 2;
                        y_incr = y_incr * 2;
                        boundL = boundL * 2;
                        boundR = boundR * 2;
                        boundT = boundT * 2;
                        boundB = boundB * 2;
                     } else {
                        x_incr = x_incr * 25 / 10;
                        y_incr = y_incr * 25 / 10;
                        boundL = boundL * 25 / 10;
                        boundR = boundR * 25 / 10;
                        boundT = boundT * 25 / 10;
                        boundB = boundB * 25 / 10;
                     }
                  } else {
                     printf ( "\nMaximum distance reached." );
                  }
                  break;
               default  :
                  changed = 0;
                  break;
            }
            if( running && changed ) {
                mandelbrot( boundL, boundR, boundT, boundB, x_incr, y_incr, max, disbuf, 0 );
                display( disbuf );
            }
         }
      }
   }

   free( disbuf );
   return;
}

