// jinx.c
//
// Written by Wayne Parham
//
// 07/14/2021

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// Compile with -DENABLESAVELOAD to enable save and load.

// Small numbers are cast as 8-bit chars when compiling with CC65.
#ifdef __CC65__
typedef unsigned char number;
#else
typedef int number;
#endif

#define SCREENWIDTH            80
#define SCREENHEIGHT           24
#define NUMLOCATIONS          143
#define MAXCARRYSIZE           30
#define KILLVALUE               7
#define FLOODDURATION         123
#define ELIMINATEWRAITHVALUE   50
#define INITIALWRAITHAGITATION  0
#define INITIALWRAITHSTRENGTH  10
#define MAXWRAITHSTRENGTH     100
#define TURNSTOAGITATE          4
#define DEFAULTAGITATEFREQUENCY 4
#define RINGAGITATEFREQUENCY    6
#define IMMOVABLE             255
#define DETAILED                1
#define RUNNING                 1
#define TRUE                    1
#define HERE                    1
#define ON                      1
#define OFF                     0
#define NONE                    0
#define FALSE                   0
#define EMPTY                   0
#define NORMAL                  0
#define NOWHERE                 0
#define NOTHING                 0
#define ILLEGAL                 0
#define UNAVAILABLE             0
#define UNINITIALIZED           0
#define HIGHWRAITHSTRENGTH     80
#define HIGHWRAITHENCOUNTERS  252 

char* s = "%s";
char* sps = " %s";
char* sCr = "%s\n";
char* spsCr = "  %s\n";
char* dot = ".";
char* sdotCr = "%s.\n";
char* dotCr = ".\n";
char* Cr = "\n";
char* inp = "\n> ";
char* quoted = "\"%s\"\n";

// Location description strings
char* housefront = "in front of the house";
char* porch = "on the porch";
char* back = "behind the building";
char* backyard = "in the back yard";
char* entryhall = "in the entry hall";
char* descentryhall = " inside the house";
char* hall = "in the hall";
char* garage = "in the garage";
char* kitchen = "in the kitchen";
char* diningroom = "in the dining room";
char* livingroom = "in the living room";
char* sittingroom = "in the sitting room";
char* stairs = "near the stairs";
char* bedroom = "in the bedroom";
char* kidsroom = "in the kid's room";
char* shelter = "in the storm shelter";
char* school = "at an old school";
char* office = "in the office";
char* apartments = "at some apartments";
char* largebuilding = "at a large building";
char* neighborhood = "in a neighborhood";
char* lamppost = "near a lamp post";
char* openfield = "in an open field";
char* grassyfield = "in a field of tall grass";
char* rockyfield = "in a graveled area";
char* openarea = "in an open area with few trees";
char* coinshop = "at a coin shop";
char* northhills = "in the north hills";
char* southroads = "at Southroads Mall";
char* elephantrun = "at Elephant Run";
char* abandonedairport = "at an abandoned airport";
char* hollidayhills = "at Holliday Hills";
char* hospital = "at a hospital";
char* techcenter = "in front of a building";
char* westdock = "on the west dock";
char* lab = "in the lab";
char* computerroom= "in the computer room";
char* farm = "on a farm";
char* easthills = "in the east hills";
char* golfcourse = "on a golf course";
char* proshop = "at the pro shop";
char* college = "at a college";
char* reception = "at the reception desk";
char* admissionsdept = "in the admissions department";
char* accountingdept = "in the accounting department";
char* campus = "on the campus";
char* fontana = "at Fontana Shopping Mall";
char* hillyland = "in some gentle hills";
char* bridge = "on a bridge";
char* forest = "in a forest";
char* forestclearing = "in a forest clearing";
char* arrowoil = "at Arrow Oil Tools";
char* shop = "in the shop";
char* walkway = "on the walkway";
char* behindshop = "behind the shop";
char* fizzypop = "at Fizzy Pop Bottling Company";
char* mountain = "on the mountain";
char* mountainarea = "on an area of the mountain";
char* northslope = "on the north slope";
char* southslope = "on the south slope";
char* eastslope = "on the east slope";
char* cliffoverlook = "on the cliff overlook";
char* church = "in an old church";
char* jinxshops = "at the Jinx shoppes";
char* jinxcentre = "at Jinx Centre";
char* southhills = "on the south hills";
char* eastedgeofhills = "at the east edge of the hills";
char* riverbank = "on the west bank of the river";
char* skylinepark = "at Skyline Park";
char* skylinelake = "at the Skyline Lake";
char* tiltawhirl = "at the Tilt-a-Whirl ride";
char* rollercoaster = "at the roller coaster";
char* belowgrate = "below the grate";
char* caveentrance = "at the cave entrance";
char* cavedwelling = "in a cave dwelling";
char* tunnel = "in a tunnel";
char* tunnelentrance = "at the tunnel entrance";
char* tunneljunction = "at the tunnel junction";
char* muddyarea = "on a muddy area";

// Additional location details
char* desc5843 = ", standing just south of house number 5843";
char* desclivrm5843 = " with shag carpet and 70s furnishings";
char* desclivrm3017 = " with a bar close to the kitchen";
char* descporch3017 = ", right by the front door to the north";
char* descyard3017 = ". There's a shelter north of here";
char* desclamppost = ". There's a notice posted here";
char* descshelter = ".\nA ladder descends into darkness";
char* descladder = ". There is a ladder attached to the wall here";
char* descrocksteps = ". Looks like someone lives here.\nThere are rocks forming upward steps";
char* desclivrm4735 = " that has been cleaned out";
char* descgarage4735 = ". There's a hatch on the floor";
char* descforestclearing = ". There is a grate on the ground";
char* desclivrm3750 = " with a bar in the corner";
char* descbdrm3750 = ". This room makes you feel uneasy";
char* descchurch = ". It has a humble but reverent altar. There are\ninteresting symbols on the altar, walls and ceiling";
char* descoutlook = ". You have an excellent view to the south and the east";
char* descsuspbridge = " with a narrow rope and wooden bridge\nrunning east to west";
char* deschillbridge = ". There's a narrow path running north and south";
char* desc3017 = ", standing just south of house number 3017";
char* desc4735 = ", standing just west of house number 4735";
char* desc3750 = ", standing just north of house number 3750";
char* desckiddypark = ". There's a kiddie park here";
char* desccampus = " with ultra-modern architecture";
char* desctallbldg = ". There's a tall building here";
char* descbldgvis = ". Buildings can be seen to the south and the west";
char* descboarded = " but everything is all boarded up";
char* desctelex = ", a factory for the Telex Corporation";
char* desctechcenter = ", a large tech center once used for commercial\nresearch";
char* descnova4 = " with a Data General Nova 4C system";
char* desceclipse = " with a Data General Eclipse C/350 system";
char* descmv8000 = " with a Data General MV/8000 system";
char* descarrowoil = " of Arrow Oil Tools. There's lots of tubular steel here";
char* descshop = ". There are CNC lathes and metal shavings on the ground";
char* descfarm = " with two large silos";
char* descfizzy = ". You see a long assembly line with\nthousands of bottles";
char* descforest = ". It is dense with tall trees everywhere";
char* desctilt = ". There are picnic tables and other rides\nnearby";
char* desccoaster = ". A section of track is missing on the main\ndownslope";
char* desclargerock = ". There's a large rock with symbols on it";
char* scattereddebris = " with some scattered debris on the ground";
char* tallbuilding = "It is very tall.\n";

// Command response and circumstance strings
char* all = "all";
char* in = " in";
char* getss = "get %s";
char* dropss = "drop %s";
char* youare = "You are %s";
char* youarecarrying = "You are carrying:\n";
char* nothing = "  nothing\n";
char* yousee = "You see:\n";
char* got = "Got %s.\n";
char* seeno = "I see no %s here.\n";
char* cannotmove = "You cannot move the %s.\n";
char* usingkey = "Using key - ";
char* ahh = "Ahh";
char* mmm = "Mmm";
char* nasty = "Nasty!";
char* nothingpalatable = "Nothing palatable here.";
char* opened = "Opened. ";
char* unlocked = "Unlocked. ";
char* needkey = "You need a key.\n";
char* notlocked = "It's not locked.\n";
char* nothingyoucancarry = "  nothing you can easily carry\n";
char* notcarrying = "Not carrying %s.\n";
char* youcango = "You can go:";
char* youcannotgo = "You can't go %s from here.\n";
char* nothingtoopen = "Nothing to open.\n";
char* nothingtoget = "Nothing to get.\n";
char* nothingtodrop = "Nothing to drop.\n";
char* nothingtoread = "Nothing to read.\n";
char* nothingtounlock = "Nothing to unlock.\n";
char* alreadyopen = "It's already open.\n";
char* ordinaryitem = "It's an ordinary %s.\n";
char* noComputer = "There's nothing here on which to type.\n";
char* noweapon = "You have no weapon.\n";
char* nopoint = "I see no point in that.\n";
char* barred = "The building is securely barred.\n";
char* toomuch = "You can't carry any more. Drop something.\n";
char* dropped = "Dropped %s.\n";
char* alreadyHave = "Already have it.\n";
char* oneatatime = "One at a time, please.\n";
char* lightmatch = "Match lit.\n";
char* lightcandle = "Used match to light candle.\n";
char* badParse = "Need clarity, please rephrase. Type 'help' if needed.\n\n";
char* printdays = " %d days";
char* printhours = " %d hours";
char* printminutes = " %d minutes";
char* openhatch = "Opening hatch - ";
char* closedhatch = "The hatch can only be opened from above.";
char* opengrate = "Opening grate - ";
char* closedgrate = "The grate can only be opened from above.\n";
char* doorlocked = "The door is locked.\n";
char* matonground = "There is a mat on the ground.\n";
char* matmoved = "Moving the mat reveals a key.\n";
char* blahblah = "Blah, blah, blah.\n";
char* nothinghappens = "Nothing happens.\n";
char* nothingfound = "I don't see it.\n";
char* frontpanelflickers = "The front-panel lights flicker";
char* diskshakes = " and the disk drive shakes.\n";
char* tapemoves = " and the tape drive jerks into motion.\n";
char* screensaysnothinghappens = "The screen says, \"Nothing happens.\"\n";
char* screensaystwiceasmuchhappens = "The screen says, \"Twice as much happens.\"\n";
char* groundshakes = "The ground shakes";
char* rainbegins = " and it begins to rain";
char* floodrain = "You're caught in a torrential downpour with high winds.\n";
char* sofaryouplayed = "\nSo far, you've played %d turns";
char* gameover = "\nGame over after %d turns.";
char* elapsedtimewas = " Elapsed time was";
char* printpointsearned = "\nYou've earned %d points out of %d possible.\n\n";
char* sureyouwanttoquit = "Are you sure you want to quit (y/n)? ";
char* building = "building";
char* xyzzy = "xyzzy";
char* wraith = "wraith";
char* wispofsmoke = "\nA wisp of smoke drifts by.";
char* strangeshadow = "\nYou catch a strange shadow from the corner of your eye.";
char* wraithattacks = "\nA wraith flies up to you and shows its bony face.";
char* wraithkillsyou = "\nA wraith flies right through your heart, killing you.";
char* deadbeef = "\nThe wraith screeches and dissipates into a thin mist.";
char* diedinflood = "The house you're inside is destroyed by the storm, taking you with it.\n";
char* voiceofGod = "\nYou feel as though you hear a voice.\n\nIt says, \"I will protect you in all things.\"\n\n";

// Item description strings
char* key = "key";
char* mat = "mat";
char* ring = "ring";
char* coin = "coin";
char* book = "book";
char* parchment = "parchment";
char* bell = "bell";
char* candle = "candle";
char* photo = "photo";
char* shoes = "shoes";
char* match = "match";
char* matches = "matches";
char* sword = "sword";
char* top = "top";
char* generator = "generator";
char* gas = "gas";
char* gasoline = "gasoline";
char* notice = "notice";
char* newspaper = "newspaper";
char* magazine = "magazine";
char* brochure = "brochure";
char* bottle = "bottle";
char* water = "water";
char* bread = "bread";
char* meat = "meat";
char* rock = "rock";
char* statue = "statue";
char* matchesRn = "box of matches";
char* breadRn = "loaf of bread";
char* meatRn = "piece of salted meat";
char* bottleRn = "bottle of water";
char* generatorRn = "electric generator";
char* gasolineRn = "can of gasoline";
char* topRn = "toy top, labeled \"BeyBlade\" Takara brand";
char* bookDesc = "It is a very old book in the form of a codex but written in an unfamiliar\nlanguage.  From the layout, it looks like a book of salutations and psalms.\nYou can pronounce the words, but you do not know what they mean.\n";
char* parchmentDesc = "The parchment has the following note:\n\nYou must have faith in God.  Demonstrate your faith in God this way.\nAt the altar, do these things in this exact order:\n\n1. Light the candle.\n2. Ring the bell.\n3. Read the book.\n4. Pray.\n\n";
char* photoDesc = "The photograph is of a very beautiful girl.\n";
char* ringDesc = "The ring has a symbol on it that looks like an \"x\" or a \"*\" but with four lines\ncrossed instead of two or three.\n";
char* rockDesc = "The rock is interesting in that it has petroglyphs carved into it.  There are\nimages and what appear to be letters that look like this:\n\n       G W N\n\n   ooooo^.? ~~~~~\n\n     y   b   y\n\n";
char* statueDesc = "The \"statue\" is a cobbled-together structure made from a ladder with all kinds\nof miscellaneous items hanging on it.  It appears that someone was trying to\nrepresent something mysterious using ordinary objects.\n";
char* swordDesc = "It's a silver sword with mystical symbols on it.\n";
char* shoesDesc = "They're pink ballet shoes.\n";
char* noticeText = "                             Beware of Wraiths!\n\nWraiths  are  deadly spectres, with skeletal faces and  shadowy,  smoky  bodies.\nThey  often look like a cloud of smoke but they can run you through even  before\nyou can react.  Be sure to carry some silver, which will kill a wraith when it's\nimpaled.\n\n";
char* magazineArticle = "                             A Blast from the Past!\n\nThe  Skyline Amusement Park had many attractions and was popular with  families,\nhaving fun activities for both children and adults alike.   It had picnic tables\nand a concession stand for snack purchases.  There was a large spring-fed  clear\nlake  with  a nice sandy beach for swimming or sunbathing.  It  even  had  pedal\npowered paddle boats and canoes.\n\nOther rides included a Tilt-A-Whirl, Stage Coach,  an electric boat ride and  an\nauthentic steam-powered train on a track that circled the lake.  It had a wooden\nroller-coaster,  advertised  as the \"The Longest Wooden Roller  Coaster  in  the\nWorld.\"\n\nSadly, the roller coaster collapsed during a ride, killing its passengers.   The\npark was closed soon after that.\n";
char* newspaperArticle = "Residents first became aware of the scenic treasure in 1907, when the World News\nreported  that  a  blasting crew working for the  Midland  Valley  Railroad  had\ndiscovered  a  cave  about  100 feet above the river on the  east  side  of  the\nmountain.\n";
char* realEstateBrochure = "\n            Welcome to the New Haven neighborhood by Holliday Hills!\n\nPlots  are  now being sold for this excellent community,  conveniently  situated\nnear busineses and shopping centers.   But land is selling fast so speak to your\nrealtor today!\n\nYou may wonder why Urbana Avenue is the widest street in the neighborhood.  It's\nbecause  before Urbana was a street it was the main runway at the now  abandoned\nCommercial Airport.\n";
char* prayer = "You can't understand the words, but you sound them out:\n\n\"Tam amavantam yajatam shuram dhamasu shavistam mitram yajai hotrabhyah\"\n";
// The prayer says, in Sanscrit (or Avestan, Yasna 10:6), "The powerful God, the most powerful Mithra in the world of the creatures, I will adore him with libations."

// Helpful text strings
char* introText = "You just  heard a strange sound and rushed outside to find someone running away.\nOr  at least you think you saw the shadow of someone.  It's hard to say  because\nthe street is dark.  All the houses are boarded up and no lights are on except a\nstreet lamp to the south that's dimly flickering.\n\n";
char* helpString = "\n\n\n\nYou  are  in a city that has been crippled by a recent event that  darkened  the\nsky, assumed to be the result of a meteor strike.  But just like in Tunguska, no\nspecific  impact has been identified.  Most everyone in the city has  evacuated,\ntaking their most valuable possessions.  You have stayed behind.\n\nYour task is to survive, and in order to do that, you gather resources where you\nfind them.  There are a few natural (and possibly supernatural) hazards that may\nimpede  you.  Your goal is to find as many resources as you can,  and  to  store\nthem in your house or any other safe house that you have access.\n\nYou  may move in any of the following directions with the \"go\"  command:  north,\nnortheast,  east, southeast, south, southwest, west, northwest, up or down.  You\ncan  also  abbreviate  the movement command and  omit the  word  \"go\"  which  is\nimplied, e.g. \"nw\" to go northwest.\n\nYou  can perform tasks such as get, put, move, open, type, say,  read,  examine,\ninventory,  score  and quit.  There are other commands you can use too,  so  try\nthings to see what works.  If a command can't be done, you'll be told.\n";

// Locations
typedef enum {
   Nowhere,                            //   0
   House5843Front,                     //   1
   House5843EntryHall,                 //   2
   House5843Garage,                    //   3
   House5843Kitchen,                   //   4
   House5843LivingRoom,                //   5
   House5843Stairs,                    //   6
   House5843Bedroom,                   //   7
   OldSchool1,                         //   8
   Apartments,                         //   9
   TelexFactory,                       //  10
   Neighborhood3,                      //  11
   LampPost,                           //  12
   OpenField6,                         //  13
   OpenField7,                         //  14
   OpenField8,                         //  15
   OpenField9,                         //  16
   OpenField10,                        //  17
   OpenField11,                        //  18
   OpenField12,                        //  19
   OpenField13,                        //  20
   OpenField14,                        //  21
   CoinShop,                           //  22
   NorthHills1,                        //  23
   NorthHills2,                        //  24
   OpenField4,                         //  25
   OpenField5,                         //  26
   SouthroadsMall,                     //  27
   Neighborhood1,                      //  28
   House3017Front,                     //  29
   House3017Porch,                     //  30
   House3017EntryHall,                 //  31
   House3017SittingRoom,               //  32
   House3017Kitchen,                   //  33
   House3017Hall,                      //  34
   House3017Bedroom,                   //  35
   House3017LivingRoom,                //  36
   House3017BackYard,                  //  37
   House3017Shelter,                   //  38
   OpenField1,                         //  39
   ElephantRunFront,                   //  40
   ElephantRunHall,                    //  41
   ElephantRunBack,                    //  42
   ElephantRunKitchen,                 //  43
   ElephantRunTables,                  //  44
   ElephantRunOffice,                  //  45
   Neighborhood2,                      //  46
   AbandonedAirport,                   //  47
   HollidayHills,                      //  48
   Hospital,                           //  49
   TechCenterWestDock,                 //  50
   TechCenterLab,                      //  51
   TechCenterOffice,                   //  52
   TechCenterComputer,                 //  53
   TechCenterFront,                    //  54
   Farm,                               //  55
   EastHills1,                         //  56
   EastHills2,                         //  57
   EastHills3,                         //  58
   EastHills4,                         //  59
   GolfCourse1,                        //  60
   GolfCourse2,                        //  61
   ProShop,                            //  62
   CollegeOfficeFront,                 //  63
   CollegeReception,                   //  64
   CollegeAdmissions,                  //  65
   CollegeAccounting,                  //  66
   CollegeCampus1,                     //  67
   CollegeCampus2,                     //  68
   CollegeCampus3,                     //  69
   FontanaShoppingMall,                //  70
   Neighborhood4,                      //  71
   House4735Front,                     //  72
   House4735EntryHall,                 //  73
   House4735LivingRoom,                //  74
   House4735Hall,                      //  75
   House4735Bedroom,                   //  76
   House4735Kitchen,                   //  77
   House4735Garage,                    //  78
   HillyLand1,                         //  79
   HillyLand2,                         //  80
   HillyLand3,                         //  81
   HillyLand4,                         //  82
   Neighborhood5,                      //  83
   OldSchool2,                         //  84
   House3750Front,                     //  85
   House3750EntryHall,                 //  86
   House3750LivingRoom,                //  87
   House3750Hall,                      //  88
   House3750KidsRoom,                  //  89
   House3750Kitchen,                   //  90
   House3750Bedroom,                   //  91
   House3750Stairs,                    //  92
   House3750Garage,                    //  93
   Bridge1,                            //  94
   Bridge2,                            //  95
   Bridge3,                            //  96
   ForestLand1,                        //  97
   ForestLand2,                        //  98
   ForestLand3,                        //  99
   ForestLand4,                        // 100
   ArrowOilBack,                       // 101
   ArrowOilShop,                       // 102
   ArrowOilOffice,                     // 103
   ArrowOilComputerRoom,               // 104
   ArrowOilFront,                      // 105
   ArrowOilWalkway,                    // 106
   FizzyPop,                           // 107
   Mountain1,                          // 108
   CaveEntrance,                       // 109
   Mountain2,                          // 110
   NorthSlope,                         // 111
   Mountain3,                          // 112
   Mountain4,                          // 113
   Church,                             // 114
   SouthSlope,                         // 115
   CliffOverLook,                      // 116
   EastSlope,                          // 117
   JinxShops,                          // 118
   JinxCentre,                         // 119
   ForestLand5,                        // 120
   ForestClearing,                     // 121
   OpenField2,                         // 122
   OpenField3,                         // 123
   SouthHills1,                        // 124
   SouthHills2,                        // 125
   EastEdgeOfHills,                    // 126
   RiverBank1,                         // 127
   RiverBank2,                         // 128
   RiverBank3,                         // 129
   SkylinePark,                        // 130
   SkylineLake,                        // 131
   TiltAWhirl,                         // 132
   RollerCoaster,                      // 133
   Below3017Shelter,                   // 134
   TunnelNorth,                        // 135
   CaveDwelling,                       // 136
   TunnelWest,                         // 137
   TunnelJunction,                     // 138
   TunnelEast,                         // 139
   Below4735Garage,                    // 140
   TunnelSouth,                        // 141
   BelowGrate,                         // 142
   LastLocation=BelowGrate
} Location_t;

char* Location[LastLocation+1];
char* Descript[LastLocation+1];
number visited[LastLocation+1];

// Items
typedef enum {
   Nothing,                            //   0
   Key,                                //   1
   Mat,                                //   2
   Ring,                               //   3
   Coin,                               //   4
   Book,                               //   5
   Parchment,                          //   6
   Bell,                               //   7
   Candle,                             //   8
   Photo,                              //   9
   Shoes,                              //  10
   Matches,                            //  11
   Sword,                              //  12
   Top,                                //  13
   Generator,                          //  14
   Gasoline,                           //  15
   Notice,                             //  16
   Newspaper,                          //  17
   Magazine,                           //  18
   Brochure,                           //  19
   Bottle,                             //  20
   Bread,                              //  21
   Meat,                               //  22
   Rock,                               //  23
   Statue,                             //  24
   LastItem=Statue
} Item_t;

char*  Item[LastItem+1];
char*  Detail[LastItem+1];
number size[LastItem+1];
number value[LastItem+1];

Item_t Inventory[LastItem+1];
Location_t locationOfItem[LastItem+1];

// Directions
typedef enum {
   North,
   Northeast,
   East,
   Southeast,
   South,
   Southwest,
   West,
   Northwest,
   Up,
   Down
} Direction_t;

char* Direction[] = {
    "north","northeast","east","southeast","south","southwest","west","northwest","up","down"
};

// Map: Given a location and a direction to move, returns the location it connects to
//      or 0 if not a valid move.  Map can change during game play.
Direction_t Move[NUMLOCATIONS][10] = {
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,    0,    0,    0,    0,    0,    0,    0,    0,    0  }, //   0 Nowhere
     {   2,   16,   19,    0,   12,    0,    8,    0,    0,    0  }, //   1 House5843Front
     {   5,    0,    6,    0,    1,    0,    3,    0,    0,    0  }, //   2 House5843EntryHall
     {   4,    0,    2,    0,    0,    0,    0,    0,    0,    0  }, //   3 House5843Garage
     {   0,    0,    5,    0,    3,    0,    0,    0,    0,    0  }, //   4 House5843Kitchen
     {   0,    0,    0,    0,    2,    0,    4,    0,    0,    0  }, //   5 House5843LivingRoom
     {   0,    0,    0,    0,    0,    0,    2,    0,    7,    0  }, //   6 House5843Stairs
     {   0,    0,    0,    0,    0,    0,    0,    0,    0,    6  }, //   7 House5843Bedroom
     {   9,   10,    1,   11,    0,    0,   46,    0,    0,    0  }, //   8 OldSchool1
     {  27,    0,   10,    0,    8,    0,   39,    0,    0,    0  }, //   9 Apartments
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  22,   13,   16,   19,    0,    8,    9,    0,    0,    0  }, //  10 TelexFactory
     {   0,   19,    0,   61,   25,    0,   47,    8,    0,    0  }, //  11 Neighborhood3
     {   1,    0,   19,    0,    0,    0,    8,   16,    0,    0  }, //  12 LampPost
     {   0,   23,   14,   17,   16,   10,   22,    0,    0,    0  }, //  13 OpenField6
     {   0,   24,   15,   18,   17,   16,   13,   23,    0,    0  }, //  14 OpenField7
     {   0,    0,   50,    0,   18,   17,   14,   24,    0,    0  }, //  15 OpenField8
     {  13,   14,   17,   20,   19,    1,   10,   22,    0,    0  }, //  16 OpenField9
     {  14,   15,   18,   21,   20,   19,   16,   13,    0,    0  }, //  17 OpenField10
     {  15,    0,    0,    0,   21,   20,   17,   14,    0,    0  }, //  18 OpenField11
     {  16,   17,   20,   70,   60,   11,    1,   10,    0,    0  }, //  19 OpenField12
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  17,   18,   21,    0,   70,   60,   19,   16,    0,    0  }, //  20 OpenField13
     {  18,    0,   56,   57,   72,   70,   20,   17,    0,    0  }, //  21 OpenField14
     {   0,    0,   13,   16,   10,    0,    0,    0,    0,    0  }, //  22 CoinShop
     {   0,    0,   24,   14,    0,   13,    0,    0,    0,    0  }, //  23 NorthHills1
     {   0,    0,    0,   15,    0,   14,   23,    0,    0,    0  }, //  24 NorthHills2
     {  11,    0,   61,    0,   26,   49,    0,    0,    0,    0  }, //  25 OpenField4
     {  25,    0,   63,   79,    0,   95,    0,    0,    0,    0  }, //  26 OpenField5
     {  28,    0,    0,    0,    9,    0,    0,    0,    0,    0  }, //  27 MidwayShops
     {   0,    0,   27,    0,   39,   42,   29,    0,    0,    0  }, //  28 Neighborhood1
     {  30,    0,   28,    0,    0,    0,    0,    0,    0,    0  }, //  29 House3017Front
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  31,    0,    0,    0,   29,    0,    0,    0,    0,    0  }, //  30 House3017Porch
     {  33,    0,   34,    0,   30,    0,   32,    0,    0,    0  }, //  31 House3017EntryHall
     {   0,    0,   31,    0,    0,    0,    0,    0,    0,    0  }, //  32 House3017SittingRoom
     {  36,    0,    0,    0,   31,    0,    0,    0,    0,    0  }, //  33 House3017Kitchen
     {   0,    0,   35,    0,    0,    0,   31,    0,    0,    0  }, //  34 House3017Hall
     {   0,    0,    0,    0,    0,    0,   34,    0,    0,    0  }, //  35 House3017Bedroom
     {   0,    0,    0,    0,   33,    0,   37,    0,    0,    0  }, //  36 House3017LivingRoom
     {  38,    0,   36,    0,    0,    0,    0,    0,    0,    0  }, //  37 House3017BackYard
     {   0,    0,    0,    0,   37,    0,    0,    0,    0,  134  }, //  38 House3017Shelter
     {  40,   28,    9,    0,   46,    0,   94,    0,    0,    0  }, //  39 OpenField1
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  41,    0,    0,    0,   39,    0,    0,    0,    0,    0  }, //  40 ElephantRunFront
     {  42,    0,   45,    0,   40,    0,   44,    0,    0,    0  }, //  41 ElephantRunHall
     {  28,    0,    0,    0,   41,    0,    0,    0,    0,    0  }, //  42 ElephantRunBack
     {   0,    0,   44,    0,    0,    0,    0,    0,    0,    0  }, //  43 ElephantRunKitchen
     {   0,    0,   41,    0,    0,    0,   43,    0,    0,    0  }, //  44 ElephantRunTables
     {   0,    0,    0,    0,    0,    0,   41,    0,    0,    0  }, //  45 ElephantRunOffice
     {  39,    0,    8,    0,   47,    0,    0,    0,    0,    0  }, //  46 Neighborhood2
     {  46,    0,   11,    0,   48,    0,    0,    0,    0,    0  }, //  47 AbandonedAirport
     {  47,    0,    0,   49,    0,    0,    0,    0,    0,    0  }, //  48 HollidayHills
     {   0,   25,    0,    0,    0,    0,    0,   48,    0,    0  }, //  49 Hospital
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,    0,   51,    0,    0,    0,   15,    0,    0,    0  }, //  50 TechCenterWestDock
     {   0,    0,   52,    0,    0,    0,   50,    0,    0,    0  }, //  51 TechCenterLab
     {  53,    0,    0,    0,   54,    0,   51,    0,    0,    0  }, //  52 TechCenterOffice
     {   0,    0,    0,    0,   52,    0,    0,    0,    0,    0  }, //  53 TechCenterComputer
     {  52,    0,    0,    0,   55,    0,    0,    0,    0,    0  }, //  54 TechCenterFront
     {  54,    0,    0,    0,   56,    0,    0,    0,    0,    0  }, //  55 Farm
     {  55,    0,    0,    0,   57,    0,   21,    0,    0,    0  }, //  56 EastHills1
     {  56,    0,    0,    0,   58,   71,   72,   21,    0,    0  }, //  57 EastHills2
     {  57,    0,    0,    0,   59,    0,    0,   71,    0,    0  }, //  58 EastHills3
     {  58,    0,    0,    0,    0,   82,   81,   80,    0,    0  }, //  59 EastHills4
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  19,   20,    0,    0,   62,    0,    0,    0,    0,    0  }, //  60 GolfCourse1
     {  62,    0,   70,   80,   63,    0,   25,   11,    0,    0  }, //  61 GolfCourse2
     {  60,    0,    0,    0,   61,    0,    0,    0,    0,    0  }, //  62 ProShop
     {  61,    0,   64,   67,   79,   68,   26,    0,    0,    0  }, //  63 CollegeOfficeFront
     {   0,    0,   65,    0,    0,    0,   63,    0,    0,    0  }, //  64 CollegeReception
     {  66,    0,    0,    0,    0,    0,   64,    0,    0,    0  }, //  65 CollegeAdmissions
     {   0,    0,    0,    0,   65,    0,    0,    0,    0,    0  }, //  66 CollegeAccounting
     {   0,    0,    0,    0,    0,    0,    0,   63,    0,    0  }, //  67 CollegeCampus1
     {   0,   63,    0,    0,   69,    0,    0,    0,    0,    0  }, //  68 CollegeCampus2
     {  68,    0,    0,    0,    0,    0,    0,    0,    0,    0  }, //  69 CollegeCampus3
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  20,   21,    0,   71,    0,    0,   61,   19,    0,    0  }, //  70 FontanaShoppingMall
     {  72,   57,    0,   58,   81,   80,    0,   70,    0,    0  }, //  71 Neighborhood4
     {  21,    0,   73,    0,   71,    0,    0,    0,    0,    0  }, //  72 House4735Front
     {   0,   77,   74,   75,    0,    0,   72,    0,    0,    0  }, //  73 House4735EntryHall
     {  77,    0,    0,    0,   75,    0,   73,    0,    0,    0  }, //  74 House4735LivingRoom
     {  74,    0,    0,    0,   76,    0,    0,   73,    0,    0  }, //  75 House4735Hall
     {  75,    0,    0,    0,    0,    0,    0,    0,    0,    0  }, //  76 House4735Bedroom
     {   0,    0,    0,    0,   74,   73,   78,    0,    0,    0  }, //  77 House4735Kitchen
     {   0,    0,   77,    0,    0,    0,    0,    0,    0,  140  }, //  78 House4735Garage
     {  63,    0,   80,    0,   83,    0,    0,   26,    0,    0  }, //  79 HillyLand1
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,   71,    0,   59,   81,   83,   79,   61,    0,    0  }, //  80 HillyLand2
     {  80,   71,   59,    0,   82,   84,   85,    0,    0,    0  }, //  81 HillyLand3
     {  81,   59,    0,    0,    0,    0,   96,    0,    0,    0  }, //  82 HillyLand4
     {  79,   80,    0,    0,   85,    0,    0,    0,    0,    0  }, //  83 Neighborhood5
     {  85,   81,    0,    0,    0,    0,    0,    0,    0,    0  }, //  84 OldSchool2
     {   0,    0,   81,    0,   86,   84,   83,    0,    0,    0  }, //  85 House3750Front
     {  85,    0,   87,    0,   88,   90,   89,    0,    0,    0  }, //  86 House3750EntryHall
     {   0,    0,    0,    0,    0,    0,   86,    0,    0,    0  }, //  87 House3750LivingRoom
     {  86,    0,   91,    0,   92,    0,   90,    0,    0,    0  }, //  88 House3750Hall
     {   0,    0,   86,    0,    0,    0,    0,    0,    0,    0  }, //  89 House3750KidsRoom
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,   86,   88,    0,    0,    0,    0,    0,    0,    0  }, //  90 House3750Kitchen
     {   0,    0,    0,    0,    0,    0,   88,    0,    0,    0  }, //  91 House3750Bedroom
     {  88,    0,    0,    0,    0,    0,    0,    0,    0,   93  }, //  92 House3750Stairs
     {   0,    0,    0,    0,    0,    0,    0,    0,   92,    0  }, //  93 House3750Garage
     {   0,    0,   39,    0,    0,    0,   99,    0,    0,    0  }, //  94 Bridge1
     {   0,   26,    0,    0,    0,  111,    0,    0,    0,    0  }, //  95 Bridge2
     {   0,   82,    0,    0,    0,  129,    0,    0,    0,    0  }, //  96 Bridge3
     {   0,    0,    0,    0,   98,    0,    0,    0,    0,    0  }, //  97 ForestLand1
     {  97,    0,    0,    0,   99,    0,    0,    0,    0,    0  }, //  98 ForestLand2
     {  98,    0,   94,    0,  100,    0,    0,    0,    0,    0  }, //  99 ForestLand3
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {  99,    0,    0,    0,  105,    0,    0,    0,    0,    0  }, // 100 ForestLand4
     { 102,    0,    0,    0,  108,    0,    0,    0,    0,    0  }, // 101 ArrowOilBack
     { 103,    0,    0,    0,  101,    0,    0,    0,    0,    0  }, // 102 ArrowOilShop
     { 105,    0,  106,    0,  102,    0,  104,    0,    0,    0  }, // 103 ArrowOilOffice
     {   0,    0,  103,    0,    0,    0,    0,    0,    0,    0  }, // 104 ArrowOilComputerRoom
     { 100,    0,    0,    0,  103,    0,    0,    0,    0,    0  }, // 105 ArrowOilFront
     {   0,    0,  107,    0,    0,    0,  103,    0,    0,    0  }, // 106 ArrowOilWalkway
     {   0,    0,    0,    0,    0,    0,  106,    0,    0,    0  }, // 107 FizzyPop
     { 101,    0,    0,  110,  109,    0,    0,    0,    0,    0  }, // 108 Mountain1
     {   0,    0,    0,  113,    0,    0,    0,  108,    0,  136  }, // 109 CaveEntrance
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,    0,    0,  111,    0,    0,    0,  108,    0,    0  }, // 110 Mountain2
     {   0,   95,    0,  127,  114,    0,    0,  110,    0,    0  }, // 111 NorthSlope
     {   0,    0,  113,    0,  115,    0,    0,    0,    0,    0  }, // 112 Mountain3
     {   0,    0,  114,    0,  116,    0,  112,  109,    0,    0  }, // 113 Mountain4
     { 111,    0,  127,    0,  117,    0,  113,    0,    0,    0  }, // 114 Church
     { 112,    0,    0,    0,  118,    0,    0,    0,    0,    0  }, // 115 SouthSlope
     { 113,    0,  117,    0,    0,    0,    0,    0,    0,    0  }, // 116 CliffOverlook
     { 114,    0,    0,    0,  121,    0,  116,    0,    0,    0  }, // 117 EastSlope
     { 115,    0,    0,  122,  119,    0,    0,    0,    0,    0  }, // 118 JinxShops
     { 118,    0,  122,  124,    0,    0,    0,    0,    0,    0  }, // 119 JinxCentre
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,    0,  121,    0,  122,    0,    0,    0,    0,    0  }, // 120 ForestLand5
     { 117,  127,  128,    0,  123,    0,  120,    0,    0,  142  }, // 121 ForestClearing
     { 120,    0,  123,    0,  124,    0,  119,  118,    0,    0  }, // 122 OpenField2
     { 121,  128,  129,  126,  125,    0,  122,    0,    0,    0  }, // 123 OpenField3
     { 122,    0,  125,    0,    0,    0,    0,  119,    0,    0  }, // 124 SouthHills1
     { 123,    0,  126,    0,    0,    0,  124,    0,    0,    0  }, // 125 SouthHills2
     {   0,  129,    0,    0,    0,    0,  125,  123,    0,    0  }, // 126 EastEdgeOfHills
     {   0,    0,    0,  128,    0,  121,  114,  111,    0,    0  }, // 127 RiverBank1
     {   0,    0,    0,  129,    0,  123,  121,  127,    0,    0  }, // 128 RiverBank2
     {   0,   96,    0,  130,    0,  126,  123,  128,    0,    0  }, // 129 RiverBank3
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     { 129,    0,    0,    0,  133,  131,  132,    0,    0,    0  }, // 130 SkyLinePark
     { 132,  130,  133,    0,    0,    0,    0,    0,    0,    0  }, // 131 SkyLineLake
     {   0,    0,  130,  133,  131,    0,    0,    0,    0,    0  }, // 132 TiltAWhirl
     { 130,    0,    0,    0,    0,    0,  131,  132,    0,    0  }, // 133 RollerCoaster
     {   0,    0,    0,    0,  135,    0,    0,    0,   38,    0  }, // 134 Below3017Shelter
     { 134,    0,    0,    0,  138,    0,    0,    0,    0,    0  }, // 135 TunnelNorth
     {   0,    0,  137,    0,    0,    0,    0,    0,  109,    0  }, // 136 CaveDwelling
     {   0,    0,  138,    0,    0,    0,  136,    0,    0,    0  }, // 137 TunnelWest
     { 135,    0,  139,    0,  141,    0,  137,    0,    0,    0  }, // 138 TunnelJunction
     {   0,    0,  140,    0,    0,    0,  138,    0,    0,    0  }, // 139 TunnelEast
//      N     NE    E     SE    S     SW    W     NW    U     D      //   =======================
     {   0,    0,    0,    0,    0,    0,  139,    0,   78,    0  }, // 140 Below4735Garage
     { 138,    0,    0,    0,  142,    0,    0,    0,    0,    0  }, // 141 TunnelSouth
     { 141,    0,    0,    0,    0,    0,    0,    0,  121,    0  }, // 142 BelowGrate
};

// WorldState bits:
// 
//  0   Game Running
//  1   Mat Moved
//  2   Wraiths Eliminated
//  3   World Flooded
//  4   Sword Found
//  5   Wraith House Entered
//
number WorldState;

// User input
char buffer[SCREENWIDTH];

// Current location
number currentLocation;

// Number of turns played
int turnsPlayed;

// Turn when flood occurs (set randomly)
int turnFlood;

// Last turn that candle was lit
int turnCandle;

// Last turn that bell was rung
int turnBell;

// Last turn that book was read
int turnBook;

// Number of minutes passed
long timeElapsed;

// Wraith encounters in the last eight moves
number wraithEncounters;

// Wraith strength
number wraithStrength;

// Wraith agitation, e.g. liklihood of attack
number wraithAgitation;

// Number of turns to increase agitation
number turnsToAgitate;

#ifdef ENABLESAVELOAD

// Save/Load File Structure
struct gameSaveState {
   number WorldState;
   number currentLocation;
   int turnsPlayed;
   int turnFlood;
   int turnCandle;
   int turnBell;
   int turnBook;
   long timeElapsed;
   number wraithEncounters;
   number wraithStrength;
   number wraithAgitation;
   number turnsToAgitate;
   int Move30N;
   int Move37N;
   int Move38S;
   int Move78D;
   int Move140U;
   int Move121D;
   int Move142U;
   number visited[LastLocation+1];
   int Inventory[LastItem+1];
   int locationOfItem[LastItem+1];
} saveState;

void loadGameState();
void saveGameState();

#endif

// ============================= Initialization ================================


void initLocations() {
// Names of Locations
   Location[1]   = housefront;                             // House5843Front
   Location[2]   = entryhall;                              // House5843EntryHall
   Location[3]   = garage;                                 // House5843Garage
   Location[4]   = kitchen;                                // House5843Kitchen
   Location[5]   = livingroom;                             // House5843LivingRoom
   Location[6]   = stairs;                                 // House5843Stairs
   Location[7]   = bedroom;                                // House5843Bedroom
   Location[8]   = school;                                 // OldSchool1
   Location[9]   = apartments;                             // Apartments
   Location[10]  = largebuilding;                          // TelexFactory
   Location[11]  = neighborhood;                           // Neighborhood3
   Location[12]  = lamppost;                               // LampPost
   Location[13]  = openfield;                              // OpenField6
   Location[14]  = openfield;                              // OpenField7
   Location[15]  = openfield;                              // OpenField8
   Location[16]  = openfield;                              // OpenField9
   Location[17]  = openfield;                              // OpenField10
   Location[18]  = openfield;                              // OpenField11
   Location[19]  = openfield;                              // OpenField12
   Location[20]  = openfield;                              // OpenField13
   Location[21]  = openfield;                              // OpenField14
   Location[22]  = coinshop;                               // CoinShop
   Location[23]  = northhills;                             // NorthHills1
   Location[24]  = northhills;                             // NorthHills2
   Location[25]  = rockyfield;                             // OpenField4
   Location[26]  = rockyfield;                             // OpenField5
   Location[27]  = southroads;                             // SouthroadsMall
   Location[28]  = neighborhood;                           // Neighborhood1
   Location[29]  = housefront;                             // House3017Front
   Location[30]  = porch;                                  // House3017Porch
   Location[31]  = entryhall;                              // House3017EntryHall
   Location[32]  = sittingroom;                            // House3017SittingRoom
   Location[33]  = kitchen;                                // House3017Kitchen
   Location[34]  = hall;                                   // House3017Hall
   Location[35]  = bedroom;                                // House3017Bedroom
   Location[36]  = livingroom;                             // House3017LivingRoom
   Location[37]  = backyard;                               // House3017BackYard
   Location[38]  = shelter;                                // House3017Shelter
   Location[39]  = grassyfield;                            // OpenField1
   Location[40]  = elephantrun;                            // ElephantRun
   Location[41]  = hall;                                   // ElephantRunHall
   Location[42]  = back;                                   // ElephantRunBack
   Location[43]  = kitchen;                                // ElephantRunKitchen
   Location[44]  = diningroom;                             // ElephantRunTables
   Location[45]  = office;                                 // ElephantRunOffice
   Location[46]  = neighborhood;                           // Neighborhood2
   Location[47]  = abandonedairport;                       // AbandonedAirport
   Location[48]  = hollidayhills;                          // HollidayHills
   Location[49]  = hospital;                               // Hospital
   Location[50]  = westdock;                               // TechCenterWestDock
   Location[51]  = lab;                                    // TechCenterLab
   Location[52]  = office;                                 // TechCenterOffice
   Location[53]  = computerroom;                           // TechCenterComputer
   Location[54]  = techcenter;                             // TechCenterFront
   Location[55]  = farm;                                   // Farm
   Location[56]  = easthills;                              // EastHills1
   Location[57]  = easthills;                              // EastHills2
   Location[58]  = easthills;                              // EastHills3
   Location[59]  = easthills;                              // EastHills4
   Location[60]  = golfcourse;                             // GolfCourse1
   Location[61]  = golfcourse;                             // GolfCourse2
   Location[62]  = proshop;                                // ProShop
   Location[63]  = college;                                // CollegeOfficeFront
   Location[64]  = reception;                              // CollegeOfficeReception
   Location[65]  = admissionsdept;                         // CollegeAdmissions
   Location[66]  = accountingdept;                         // CollegeAccounting
   Location[67]  = campus;                                 // CollegeCampus1   
   Location[68]  = campus;                                 // CollegeCampus2   
   Location[69]  = campus;                                 // CollegeCampus3   
   Location[70]  = fontana;                                // FontanaShoppingMall
   Location[71]  = neighborhood;                           // Neighborhood4
   Location[72]  = housefront;                             // House4735Front
   Location[73]  = entryhall;                              // House4735EntryHall
   Location[74]  = livingroom;                             // House4735LivingRoom
   Location[75]  = hall;                                   // House4735Hall
   Location[76]  = bedroom;                                // House4735Bedroom
   Location[77]  = kitchen;                                // House4735Kitchen
   Location[78]  = garage;                                 // House4735Garage
   Location[79]  = hillyland;                              // HillyLand1
   Location[80]  = hillyland;                              // HillyLand2
   Location[81]  = hillyland;                              // HillyLand3
   Location[82]  = hillyland;                              // HillyLand4
   Location[83]  = neighborhood;                           // Neighborhood5
   Location[84]  = school;                                 // OldSchool2
   Location[85]  = housefront;                             // House3750Front
   Location[86]  = entryhall;                              // House3750EntryHall
   Location[87]  = livingroom;                             // House3750LivingRoom
   Location[88]  = hall;                                   // House3750Hall
   Location[89]  = kidsroom;                               // House3750KidsRoom
   Location[90]  = kitchen;                                // House3750Kitchen
   Location[91]  = bedroom;                                // House3750Bedroom
   Location[92]  = stairs;                                 // House3750Stairs
   Location[93]  = garage;                                 // House3750Garage
   Location[94]  = bridge;                                 // Bridge1
   Location[95]  = bridge;                                 // Bridge2
   Location[96]  = bridge;                                 // Bridge3
   Location[97]  = forest;                                 // ForestLand1
   Location[98]  = forest;                                 // ForestLand2
   Location[99]  = forest;                                 // ForestLand3
   Location[100] = forest;                                 // ForestLand4
   Location[101] = behindshop;                             // ArrowOilBack
   Location[102] = shop;                                   // ArrowOilShop
   Location[103] = office;                                 // ArrowOilOffice
   Location[104] = computerroom;                           // ArrowOilComputerRoom
   Location[105] = arrowoil;                               // ArrowOilFront
   Location[106] = walkway;                                // ArrowOilWalkway
   Location[107] = fizzypop;                               // FizzyPop
   Location[108] = mountain;                               // Mountain1
   Location[109] = caveentrance;                           // CaveEntrance
   Location[110] = mountainarea;                           // Mountain2
   Location[111] = northslope;                             // NorthSlope
   Location[112] = mountain;                               // Mountain3
   Location[113] = mountain;                               // Mountain4
   Location[114] = church;                                 // Church
   Location[115] = southslope;                             // SouthSlope
   Location[116] = cliffoverlook;                          // CliffOverLook
   Location[117] = eastslope;                              // EastSlope
   Location[118] = jinxshops;                              // JinxShops
   Location[119] = jinxcentre;                             // JinxCentre
   Location[120] = forest;                                 // ForestLand5
   Location[121] = forestclearing;                         // ForestClearing
   Location[122] = openarea;                               // OpenField2
   Location[123] = openarea;                               // OpenField3
   Location[124] = southhills;                             // SouthHills1
   Location[125] = southhills;                             // SouthHills2
   Location[126] = eastedgeofhills;                        // EastEdgeOfHills
   Location[127] = riverbank;                              // RiverBank1
   Location[128] = riverbank;                              // RiverBank2
   Location[129] = riverbank;                              // RiverBank3
   Location[130] = skylinepark;                            // SkylinePark
   Location[131] = skylinelake;                            // SkylineLake
   Location[132] = tiltawhirl;                             // TiltAWhirl
   Location[133] = rollercoaster;                          // RollerCoaster
   Location[134] = tunnelentrance;                         // Below3017Shelter
   Location[135] = tunnel;                                 // TunnelNorth
   Location[136] = cavedwelling;                           // CaveDwelling
   Location[137] = tunnel;                                 // TunnelWest
   Location[138] = tunneljunction;                         // TunnelJunction
   Location[139] = tunnel;                                 // TunnelEast
   Location[140] = tunnelentrance;                         // Below4735Garage
   Location[141] = tunnel;                                 // TunnelSouth
   Location[142] = belowgrate;                             // BelowGrate

// Additional descriptive info
   Descript[1]   = desc5843;                               // House5843Front
   Descript[2]   = descentryhall;                          // House5843EntryHall
   Descript[5]   = desclivrm5843;                          // House5843LivingRoom
   Descript[10]  = desctelex;                              // TelexFactory
   Descript[11]  = descboarded;                            // Neighborhood3
   Descript[12]  = desclamppost;                           // LampPost
   Descript[13]  = descbldgvis;                            // OpenField6
   Descript[14]  = descbldgvis;                            // OpenField7
   Descript[15]  = descbldgvis;                            // OpenField8
   Descript[16]  = descbldgvis;                            // OpenField9
   Descript[17]  = descbldgvis;                            // OpenField10
   Descript[18]  = descbldgvis;                            // OpenField11
   Descript[19]  = descbldgvis;                            // OpenField12
   Descript[20]  = descbldgvis;                            // OpenField13
   Descript[21]  = descbldgvis;                            // OpenField14
   Descript[28]  = descboarded;                            // Neighborhood1
   Descript[29]  = desc3017;                               // House3017Front
   Descript[30]  = descporch3017;                          // House3017Porch
   Descript[31]  = descentryhall;                          // House3017EntryHall
   Descript[36]  = desclivrm3017;                          // House3017LivingRoom
   Descript[37]  = descyard3017;                           // House3017BackYard
   Descript[38]  = descshelter;                            // House3017Shelter
   Descript[46]  = descboarded;                            // Neighborhood2
   Descript[50]  = desctechcenter;                         // TechCenterWestDock
   Descript[53]  = descmv8000;                             // TechCenterComputer
   Descript[54]  = desctechcenter;                         // TechCenterFront
   Descript[55]  = descfarm;                               // TechCenterFront
   Descript[56]  = deschillbridge;                         // EastHills1
   Descript[57]  = deschillbridge;                         // EastHills2
   Descript[58]  = deschillbridge;                         // EastHills3
   Descript[66]  = desceclipse;                            // CollegeAccounting
   Descript[67]  = desccampus;                             // CollegeCampus1
   Descript[69]  = desctallbldg;                           // CollegeCampus3
   Descript[71]  = descboarded;                            // Neighborhood4
   Descript[72]  = desc4735;                               // House4735Front
   Descript[74]  = desclivrm4735;                          // House4735LivingRoom
   Descript[78]  = descgarage4735;                         // House4735Garage
   Descript[81]  = desckiddypark;                          // HillyLand3
   Descript[83]  = descboarded;                            // Neighborhood5
   Descript[85]  = desc3750;                               // House3750Front
   Descript[86]  = descentryhall;                          // House3750EntryHall
   Descript[87]  = desclivrm3750;                          // House3750LivingRoom
   Descript[91]  = descbdrm3750;                           // House3750Bedroom
   Descript[97]  = descforest;                             // ForestLand1
   Descript[98]  = descforest;                             // ForestLand2
   Descript[99]  = descforest;                             // ForestLand3
   Descript[100] = descforest;                             // ForestLand4
   Descript[101] = descarrowoil;                           // ArrowOilBack
   Descript[102] = descshop;                               // ArrowOilShop
   Descript[104] = descnova4;                              // ArrowOilComputerRoom
   Descript[107] = descfizzy;                              // FizzyPop
   Descript[110] = desclargerock;                          // Mountain2
   Descript[112] = descsuspbridge;                         // Mountain3
   Descript[113] = descsuspbridge;                         // Mountain4
   Descript[114] = descchurch;                             // Church
   Descript[116] = descoutlook;                            // CliffOverlook
   Descript[121] = descforestclearing;                     // ForestClearing
   Descript[132] = desctilt;                               // TiltAWhirl
   Descript[133] = desccoaster;                            // RollerCoaster
   Descript[134] = descladder;                             // Below3017Shelter
   Descript[136] = descrocksteps;                          // CaveDwelling
   Descript[140] = descladder;                             // Below4735Garage

// Start location
   currentLocation = House5843Front;
   visited[1] = TRUE;
}

void initItems() {
// Names of items
   Item[1]   = key;
   Item[2]   = mat;
   Item[3]   = ring;
   Item[4]   = coin;
   Item[5]   = book;
   Item[6]   = parchment;
   Item[7]   = bell;
   Item[8]   = candle;
   Item[9]   = photo;
   Item[10]  = shoes;
   Item[11]  = matches;
   Item[12]  = sword;
   Item[13]  = top;
   Item[14]  = generator;
   Item[15]  = gasoline;
   Item[16]  = notice;
   Item[17]  = newspaper;
   Item[18]  = magazine;
   Item[19]  = brochure;
   Item[20]  = bottle;
   Item[21]  = bread;
   Item[22]  = meat;
   Item[23]  = rock;
   Item[24]  = statue;

// Additional details
   Detail[3]  = ringDesc;                                  // Ring
   Detail[5]  = bookDesc;                                  // Book
   Detail[6]  = parchmentDesc;                             // Parchment
   Detail[9]  = photoDesc;                                 // Photo
   Detail[10] = shoesDesc;                                 // Shoes
   Detail[12] = swordDesc;                                 // Sword
   Detail[16] = noticeText;                                // Notice
   Detail[17] = newspaperArticle;                          // Newspaper
   Detail[18] = magazineArticle;                           // Magazine
   Detail[19] = realEstateBrochure;                        // Brochure
   Detail[23] = rockDesc;                                  // Rock
   Detail[24] = statueDesc;                                // Statue

// Size of items
   size[1]   = 0;                                          // Key
   size[2]   = 2;                                          // Mat
   size[3]   = 1;                                          // Ring
   size[4]   = 1;                                          // Coin
   size[5]   = 1;                                          // Book
   size[6]   = 0;                                          // Parchment
   size[7]   = 5;                                          // Bell
   size[8]   = 2;                                          // Candle
   size[9]   = 1;                                          // Photo
   size[10]  = 2;                                          // Shoes
   size[11]  = 0;                                          // Matches
   size[12]  = 10;                                         // Sword
   size[13]  = 1;                                          // Top
   size[14]  = 25;                                         // Generator
   size[15]  = 10;                                         // Gasoline
   size[16]  = 0;                                          // Notice
   size[17]  = 0;                                          // Newspaper
   size[18]  = 0;                                          // Magazine
   size[19]  = 0;                                          // Brochure
   size[20]  = 1;                                          // Bottle
   size[21]  = 1;                                          // Bread
   size[22]  = 1;                                          // Meat
   size[23]  = 255;                                        // Rock
   size[24]  = 255;                                        // Statue

// Value of items
   value[1]   = 3;                                         // Key
   value[2]   = 0;                                         // Mat
   value[3]   = 15;                                        // Ring
   value[4]   = 5;                                         // Coin
   value[5]   = 10;                                        // Book
   value[6]   = 2;                                         // Parchment
   value[7]   = 10;                                        // Bell
   value[8]   = 10;                                        // Candle
   value[9]   = 3;                                         // Photo
   value[10]  = 2;                                         // Shoes
   value[11]  = 2;                                         // Matches
   value[12]  = 15;                                        // Sword
   value[13]  = 1;                                         // Top
   value[14]  = 25;                                        // Generator
   value[15]  = 10;                                        // Gasoline
   value[16]  = 0;                                         // Notice
   value[17]  = 0;                                         // Newspaper
   value[18]  = 0;                                         // Magazine
   value[19]  = 0;                                         // Brochure
   value[20]  = 6;                                         // Bottle
   value[21]  = 3;                                         // Bread
   value[22]  = 3;                                         // Meat
   value[23]  = 0;                                         // Rock
   value[24]  = 0;                                         // Statue

// Location of items
   locationOfItem[1]  = Nowhere;                           // Key
   locationOfItem[2]  = House3017Porch;                    // Mat
   locationOfItem[3]  = ForestLand2;                       // Ring
   locationOfItem[4]  = CoinShop;                          // Coin
   locationOfItem[5]  = Church;                            // Book
   locationOfItem[6]  = Nowhere;                           // Parchment
   locationOfItem[7]  = Nowhere;                           // Bell
   locationOfItem[8]  = House3750Bedroom;                  // Candle
   locationOfItem[9]  = House3017SittingRoom;              // Photo
   locationOfItem[10] = House3017Bedroom;                  // Shoes
   locationOfItem[11] = Nowhere;                           // Matches
   locationOfItem[12] = Nowhere;                           // Sword
   locationOfItem[13] = House3750KidsRoom;                 // Top
   locationOfItem[14] = FizzyPop;                          // Generator
   locationOfItem[15] = Nowhere;                           // Gasoline
   locationOfItem[16] = LampPost;                          // Notice
   locationOfItem[17] = Nowhere;                           // Newspaper
   locationOfItem[18] = Nowhere;                           // Magazine
   locationOfItem[19] = Nowhere;                           // Brochure
   locationOfItem[20] = ElephantRunKitchen;                // Bottle
   locationOfItem[21] = ElephantRunKitchen;                // Bread
   locationOfItem[22] = ElephantRunKitchen;                // Meat
   locationOfItem[23] = Mountain2;                         // Rock
   locationOfItem[24] = JinxCentre;                        // Statue
}


// ================================= Functions =================================


void clearScreen() {                                                 // Clear screen
   number i;

   for( i = 0; i < SCREENHEIGHT; i++ )
      printf( Cr );
}


void bufferToLower() {                                               // Convert input buffer to lowercase
   number i;

   for( i=0; buffer[i]; i++ )
      buffer[i] = tolower( buffer[i] );
}


void setState( number state ) {                                      // Sets WorldState bit
   WorldState |= (1 << state);
}


void clearState( number state ) {                                    // Clears WorldState bit
   WorldState &= ~(1 << state);
}


number getState( number state ) {                                    // Get WorldState bit
   if( ( (WorldState) >> (state) ) & 1 )
      return( TRUE );
   else
      return( FALSE );
}


number running() {                                                   // Check to see if game over
   return( getState(0) );
}


number in16bitComputerRoom() {                                       // True in 16-bit computer room
   if( (currentLocation == 66) || (currentLocation == 104) )
      return( TRUE );                                                // College has Eclipse C/350
   else                                                              // and Arrow Oil has Nova 4C
      return( FALSE );
}


number in32bitComputerRoom() {                                       // True in 32-bit computer room
   if( currentLocation == 53 )
      return( TRUE );                                                // Tech Center has MV/8000
   else
      return( FALSE );
}


number inComputerRoom() {                                            // True in computer room
   if( in16bitComputerRoom() || in32bitComputerRoom() )
      return( TRUE );
   else
      return( FALSE );
}


number inHouse(number location) {                                    // True if here:
   if(   (location <   8)                          ||                // House 5843
       ( (location >  30) && (location <  37) )    ||                // House 3017
       ( (location >  72) && (location <  79) )    ||                // House 4735
       ( (location >  85) && (location <  94) )                      // House 3750
     )
      return( TRUE );
   else
      return( FALSE );
}


number isActiveBusiness(number location) {                           // True if here:
   if( ( (location >  39) && (location <  46) )    ||                // Elephant Run
       ( (location >  49) && (location <  55) )    ||                // Tech Center
       ( (location >  62) && (location <  70) )    ||                // College
       ( (location > 100) && (location < 108) )                      // Arrow Oil
     )
      return( TRUE );
   else
      return( FALSE );
}


number Indoors(number location) {                                    // True if indoors
   if( isActiveBusiness(location) || inHouse(location) ) 
      return( TRUE );
   else
      return( FALSE );
}


number isAbandonedBusiness() {                                       // True if here:
   if( (currentLocation ==  48) ||                                   // Holiday Hills
       (currentLocation ==  27) ||                                   // Southroads Mall
       (currentLocation ==  80) ||                                   // Fontana
       (currentLocation ==  22) ||                                   // Coin Shop
       (currentLocation ==   9) ||                                   // Apartments
       (currentLocation ==   8) ||                                   // OldSchool
       (currentLocation ==  49) ||                                   // Hospital
       (currentLocation ==  10) ||                                   // Telex
       (currentLocation ==  62) ||                                   // ProShop
       (currentLocation ==  84) ||                                   // OldSchool
       (currentLocation == 118) )                                    // JinxShops
      return( TRUE );
   else
      return( FALSE );
}


void printElapsedTime() {                                            // Print elapsed time
   number days, hours, minutes;

   days = timeElapsed / 24 / 60;
   hours = timeElapsed / 60 % 24;
   minutes = timeElapsed % 60;

   if( days > 0 )
      printf( printdays, days );
   if( hours > 0 )
      printf( printhours, hours );
   if( minutes > 0 )
      printf( printminutes, minutes );
}


number chance() {                                                    // Get random number betwen 0 and 99
   return( rand() % 100 );
}


void distribute() {                                                  // Distribute randomly-placed items

   locationOfItem[6]  = (chance() / 2) + 92;                         // Parchment
   locationOfItem[7]  = (chance() / 2) + 92;                         // Bell
   locationOfItem[11] = chance() + 42;                               // Matches
   locationOfItem[12] = chance() + 42;                               // Sword
   locationOfItem[15] = chance() + 42;                               // Gasoline
   locationOfItem[17] = (chance() / 6) + 117;                        // Newspaper
   locationOfItem[18] = (chance() / 6) + 97;                         // Magazine
   locationOfItem[19] = (chance() / 9) + 38;                         // Brochure

   turnFlood = (rand() % 333) + 66;                                  // Flood occurs
}


char* itemName(number i) {                                           // Returns item name
   if( i == 11 )
      return( matchesRn );
   else if( i == 13 )
      return( topRn );
   else if( i == 14 )
      return( generatorRn );
   else if( i == 15 )
      return( gasolineRn );
   else if( i == 20 )
      return( bottleRn );
   else if( i == 21 )
      return( breadRn );
   else if( i == 22 )
      return( meatRn );
   else
      return( Item[i] );
}


void showItemsHere(number verbose) {
   number i, here = FALSE;
  
   for( i = 1; i <= LastItem; i++ ) {
      if( locationOfItem[i] == currentLocation ) {
         here = TRUE;
      }
   }

   if( here || verbose )
      printf( yousee );
   for( i = 1; i <= LastItem; i++ ) {
      if( locationOfItem[i] == currentLocation ) {
         printf( spsCr, Item[i] );
      }
   }
   if( !here && verbose )
      printf( nothingyoucancarry );
   
   return;
}


void showWhereYouCanGo() {
   number i, loc;

   printf( youcango );

   for( i = North; i <= Down; i++ ) {
      loc = Move[currentLocation][i];
      if( loc != NOWHERE ) {
         printf( sps, Direction[i] );
      }
   }
   printf( Cr );
}


void checkAreas() {                                                  // Check for large areas treated
   number i;                                                         // as a single location

   if( (currentLocation > 12) && (currentLocation < 22) ) {          // Open field
      for( i = 13; i < 22; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 22) && (currentLocation < 25) ) {     // North hills
      for( i = 23; i < 25; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 24) && (currentLocation < 27) ) {     // Rocky fields
      for( i = 25; i < 27; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 55) && (currentLocation < 60) ) {     // East hills
      for( i = 56; i < 60; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 59) && (currentLocation < 62) ) {     // Golf course
      for( i = 60; i < 62; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 78) && (currentLocation < 83) ) {     // Hilly land
      for( i = 79; i < 83; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 96) && (currentLocation < 101) ) {    // Forest
      for( i = 97; i < 101; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 111) && (currentLocation < 114) ) {   // Mountain
      for( i = 112; i < 114; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 121) && (currentLocation < 124) ) {   // Open area
      for( i = 122; i < 124; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 123) && (currentLocation < 126) ) {   // South hills
      for( i = 124; i < 126; i++ ) {
         visited[i] = TRUE;
      }
   }
   else if( (currentLocation > 126) && (currentLocation < 130) ) {   // River bank
      for( i = 127; i < 130; i++ ) {
         visited[i] = TRUE;
      }
   }
}


number carryLoad() {                                                 // Tells how much you're carrying
   number i, s = 0;

   for( i = 0; i <= LastItem; i++ ) {
      if( Inventory[i] != NOTHING ) {
         s += size[Inventory[i]];
      }
   }
   return( s );
}


number matNotMoved() {                                               // Check to see if the mat was ever moved
   return( !getState(1) );
}


void matMoved() {                                                    // Mat has been moved
   setState(1);
   locationOfItem[Key] = House3017Porch;
   printf( s, matmoved );
}


number wraithsEliminated() {                                         // Check to see if the wraiths have been eliminated
   return( getState(2) );
}


void eliminateWraiths() {                                            // Eliminate all wraiths
   setState(2);
}


number flooded() {                                                   // Check to see if the world was ever flooded
   return( getState(3) );
}


void setFlooded() {                                                  // Indicate that the world has been flooded
   setState(3);
   if( wraithAgitation < 60 )
      wraithAgitation += 10;
}


number swordFound() {                                                // Check to see if sword found
   return( getState(4) );
}


void foundSword() {                                                  // Sword has been moved
   setState(4);
   if( wraithAgitation < 60 )
      wraithAgitation += 10;
}


number goneInRoom() {                                                // Check to see if we've ever been in wraith house
   return( getState(5) );
}


void enterRoom() {                                                   // Went into wraith house
   setState(5);
   if( wraithAgitation < 60 )
      wraithAgitation += 10;
}


number wraithAttacking() {                                           // Check to see if Wraith is attacking
   if( wraithEncounters & 128 )
      return( TRUE );
   else
      return( FALSE );
}


void havingWraithEncounter(number encounter) {                       // Record the wraith encounter event
   wraithEncounters = (wraithEncounters / 2) + (encounter * 128);
}


void unlock3017FrontDoor() {                                         // Unlock House 3017 Front Door
   Move[30][North] = 31;
   printf( s, unlocked );
}


number is3017FrontDoorUnlocked() {                                   // See if front door is unlocked
   if( Move[30][North] == 31 )
      return( TRUE );
   else
      return( FALSE );
}


void unlock3017Shelter() {                                           // Unlock House 3017 Shelter
   Move[37][North] = 38;
   Move[38][South] = 37;
   printf( s, unlocked );
}


number is3017ShelterUnlocked() {                                     // See if shelter is unlocked
   if( Move[37][North] == 38 )
      return( TRUE );
   else
      return( FALSE );
}


void open4735Hatch() {                                               // Open House 4735 garage hatch
   Move[78][Down] = 140;
   Move[140][Up] = 78;
   printf( s, opened );
}


number isHatchOpen() {                                               // See if hatch is open
   if( Move[78][Down] == 140 )
      return( TRUE );
   else
      return( FALSE );
}


void openForestGrate() {                                             // Open forest grate
   Move[121][Down] = 142;
   Move[142][Up] = 121;
   printf( s, opened );
}


number isGrateOpen() {                                               // See if grate is open
   if( Move[121][Down] == 142 )
      return( TRUE );
   else
      return( FALSE );
}


number carryingItem(char *item) {                                    // True if carrying an item
   number i;

   for( i = 0; i <= LastItem; i++ ) {
      if( (Inventory[i] != NOTHING) && (!strcasecmp(Item[Inventory[i]], item)) )
         return( TRUE );
   }
   return( FALSE );
}


number itemIsHere(char *item) {                                      // True if item at current location
   number i;

   for( i = 1; i <= LastItem; i++ ) {
      if( !strcasecmp(item, Item[i]) ) {
         if( locationOfItem[i] == currentLocation ) {
            return( TRUE );
         } else {
            return( FALSE );
         }
      }
   }
   return( FALSE );
}


int maxPoints() {                                                    // Maxiumum score possible
   number i;
   int points = 0;

   for( i = 1; i <= LastItem; i++ ) {
      points += (value[i] * 2);
   }
   points += ELIMINATEWRAITHVALUE;

   return( points );
}


int pointsEarned() {                                                 // Current score
   number i;
   int points = 0;

   for( i = 0; i <= LastItem; i++ ) {
      if( Inventory[i] != NOTHING ) {                                // Carrying item
         points += value[Inventory[i]];                              //     or
      } else if( inHouse(locationOfItem[i]) && visited[locationOfItem[i]] ) {
         points += ( value[i] * 2 );                                 // Item in house
      }
   }

   points += wraithsEliminated() * ELIMINATEWRAITHVALUE;             // Bonus for eliminating Wraiths                              

   return( points );
}


void increaseAgitation() {                                           // Increase Wraith agitation
   if( carryingItem(ring) )
      turnsToAgitate = RINGAGITATEFREQUENCY;
   else
      turnsToAgitate = DEFAULTAGITATEFREQUENCY;
   if( ! (turnsPlayed % turnsToAgitate) )
      wraithAgitation++;
}


void killWraith() {                                                  // Kill Wraith with Sword
   havingWraithEncounter( FALSE );
   if( wraithStrength > KILLVALUE )
      wraithStrength = wraithStrength - KILLVALUE;
   if( wraithAgitation > KILLVALUE )
      wraithAgitation = wraithAgitation - KILLVALUE;
   printf( sCr, deadbeef );
}


void GameOver() {                                                    // Set WorldState to "game over"
   clearState(0);
   printf( gameover, turnsPlayed );
   if( timeElapsed > 0 ) {
      printf( elapsedtimewas );
      printElapsedTime();
      printf( dot );
   }
   printf( printpointsearned, pointsEarned(), maxPoints() );
}


void flood() {                                                       // Flood the World
   number i;

   printf( s, floodrain );
   timeElapsed += FLOODDURATION;
   setFlooded();

   if( (currentLocation > 28) && (currentLocation < 38) ) {          // If you're inside
      printf( s, diedinflood );                                      // House 3017, you're dead
      GameOver();
      return;
   }

   for( i = 29; i < 38; i++ ) {                                      // Remove House 3017
      Location[i] = muddyarea;
      Descript[i] = NULL;
      visited[i] = FALSE;
   }
   Move[30][North] = 31;

   Descript[31] = scattereddebris;                                   // litter with debris

   for( i = 1; i <= LastItem; i++ ) {                                // and remove items
      if( (locationOfItem[i] > 28) && (locationOfItem[i] < 38)  ) {
         locationOfItem[i] = NOWHERE;
      }
   }
}


void wraithAction() {                                                // Now it's the Wraith's turn
   number c;

   if( wraithsEliminated() )
      return;

   c = chance();
   if( c < wraithAgitation ) {
      havingWraithEncounter( TRUE );
      c = chance();
      if( wraithStrength < HIGHWRAITHSTRENGTH ) {
         if( (c > 60) && (c < 80) )
            printf( sCr, strangeshadow );
         else if( c > 79 )
            printf( sCr, wispofsmoke );
         if( wraithStrength < MAXWRAITHSTRENGTH )
            wraithStrength++;
      }
      else {
         if( wraithEncounters > HIGHWRAITHENCOUNTERS ) {
            printf( sCr, wraithkillsyou );
            GameOver();
         }
         else {
            printf( sCr, wraithattacks );
            if( wraithStrength < MAXWRAITHSTRENGTH )
               wraithStrength++;
         }
      }
   }
   else {
      havingWraithEncounter( FALSE );
      increaseAgitation();
   }
}


void checkFate() {                                                   // External Events
   int seed;

   if( strlen(buffer) == 0 )
      return;

   if( turnsPlayed == 11 ) {                                         // Distribute randomly-placed items
      seed = ( (timeElapsed+1) * (pointsEarned()+1) );
      srand( seed );
      distribute();
   }

   if( !flooded()    &&                                              // Check the weather
     ( turnFlood != UNINITIALIZED) && (turnsPlayed == turnFlood) ) {
      flood();
   }

   if( running() )
      wraithAction();                                                // See what the Wraiths are doing
}


number haveKey() {                                                   // Do we have the key?
   return( carryingItem( key ) );
}


void printLocation(number verbose) {                                 // Print current location
   if( visited[currentLocation] && !verbose ) {
      printf( sCr, Location[currentLocation] );
   } else {
      printf( youare, Location[currentLocation] );
      if( Descript[currentLocation] )
         printf( s, Descript[currentLocation] );
      printf( dotCr );
   }
}


// ============================= Character Actions =============================
 

void attack() {                                                      // Attack (kill)
   bufferToLower();
   if( strlen( buffer ) < 6 ) {
      printf( s, badParse );
      return;
   }
   turnsPlayed++;
   timeElapsed++;
   if( strstr(buffer, wraith) ) {
      if( ! wraithAttacking() ) {
         printf( seeno, wraith );
         return;
      }
      else {
         if( carryingItem(sword) ) {
            killWraith();
         }
         else {
            printf( s, noweapon );
         }
      }
      return;
   }
   printf( s, nopoint );
}


void drink() {                                                       // Drink
   bufferToLower();
   if( carryingItem(bottle) || itemIsHere(bottle) ) { 
      if( (strlen(buffer) == 5) || (strstr(buffer, water)) ) {
         printf( sCr, ahh );
      } else {
         printf( sCr, nasty );
      }
   }
   else {
      printf( sCr, nothingpalatable );
   }
}


void drop() {                                                        // Drop item
   number i;
   char *sp;
   char *item;

   sp = strchr(buffer, ' ');
   if (sp == NULL) {
      printf( badParse );
      return;
   }

   item = sp + 1;
   if( strstr(item, gas) )
      item = gasoline;
   else if( strstr(item, match) )
      item = matches;

   turnsPlayed++;

   for( i = 0; i <= LastItem; i++ ) {                                // scan for item in inventory
      if( (Inventory[i] != NOTHING) && (!strcasecmp(Item[Inventory[i]], item)) ) {
         locationOfItem[Inventory[i]] = currentLocation;             // put item in current location
         Inventory[i] = NOTHING;                                     // remove from inventory
         printf( dropped, item );
         return;
      }
   }

   printf( notcarrying, item );
}


void dropall() {                                                     // Drop everything
   number i, haveitems = FALSE;

   for( i = 0; i <= LastItem; i++ ) {
      if( (Inventory[i] != NOTHING) ) {
         sprintf( buffer, dropss, Item[Inventory[i]] );
         drop();
         haveitems = TRUE;
      }
   }
   if( !haveitems )
      printf( notcarrying, "anything" ); 
}


void eat() {                                                         // Eat
   bufferToLower();
   if( carryingItem(bread) || itemIsHere(bread) || carryingItem(meat) || itemIsHere(meat) ) { 
      if( (strlen(buffer) == 3) || (strstr(buffer, bread)) || (strstr(buffer, meat)) ) {
         printf( sCr, mmm );
      } else {
         printf( sCr, nasty );
      }
   }
   else {
      printf( sCr, nothingpalatable );
   }
}


void examine() {                                                     // Examine item
   number i;
   char *sp;
   char *item;

   sp = strchr(buffer, ' ');
   if( sp == NULL ) {
      printf( badParse );
      return;
   }

   item = sp + 1;

   if( !strcasecmp(item, all) ) {                                    // disallow "examine all"
      printf( s, oneatatime );
      return;
   }

   turnsPlayed++;
   timeElapsed++;

   if( (currentLocation == 69) && (strstr(buffer, building)) ) {
      printf( s, tallbuilding );
      return;
   }

   for( i = 0; i <= LastItem; i++ ) {                                // scan for item in inventory
      if( (Inventory[i] != NOTHING) && (!strcasecmp(Item[Inventory[i]], item)) ) {
         if( Detail[Inventory[i]] ) {
            printf( s, Detail[Inventory[i]] );                       // print item detail, if there is any
	 } else {
            printf( ordinaryitem, itemName(Inventory[i]) );          // else print item name
         }
         return;
      }
   }

   for( i = 1; i <= LastItem; i++ ) {                                // see if it's at the current location
      if( !strcasecmp(item, Item[i]) ) {
         if( locationOfItem[i] == currentLocation ) {
            if( Detail[i] ) {
               printf( s, Detail[i] );                               // print item detail, if there is any
	    } else {
               printf( ordinaryitem, itemName(i) );                  // else print item name
            }
            return;
         }
      }
   }

   printf( nothingfound );                                           // item not found
}


void get() {                                                         // Get item
   number i, j;
   char *sp;
   char *item;

   sp = strchr( buffer, ' ' );
   if( sp == NULL ) {
      printf( badParse );
      return;
   }

   item = sp + 1;
   if( strstr(item, gas) )
      item = gasoline;
   else if( strstr(item, match) )
      item = matches;

   if( carryingItem(item) ) {                                        // if we already have it, say so
      printf( alreadyHave );
      return;
   }

   turnsPlayed++;

   for( i = 1; i <= LastItem; i++ ) {                                // scan for item
      if( !strcasecmp(item, Item[i]) ) {
         if( locationOfItem[i] == currentLocation ) {                // if the item is here
            if( size[i] == IMMOVABLE ) {
               printf( cannotmove, item );
               if( i == Statue ) {
                  printf( s, groundshakes );
                  if( !flooded() ) {
                     printf( sdotCr, rainbegins );
                     flood();
                  }
                  else {
                     printf( dotCr );
                  }
               }
               return;
            }
            else if( size[i] + carryLoad() > MAXCARRYSIZE ) {        // and not too much to carry
               printf( s, toomuch );
               return;
            }
            else {
               if( (i == Notice) && (currentLocation == 12) ) {
                  Descript[12] = NULL;
               }
               for( j = 0; j <= LastItem; j++ ) {
                  if( Inventory[j] == NOTHING ) {
                     Inventory[j] = i;                               // put item in inventory
                     locationOfItem[i] = NOWHERE;                    // remove item from current location
                     printf( got, item );
                     if( !strcasecmp(item, mat) && matNotMoved() ) {
                        matMoved();
                     }
                     if( (i == Sword) && !swordFound() ) {           // sword increases wraith agitation
                        foundSword();
                     }
                     return;
                  }
               }
            }
         }
      }
   }
   if( isActiveBusiness( currentLocation ) )
      printf( s, nopoint );
   else
      printf( seeno, item );
}


void getall() {                                                      // Get Everything
   number i, itemsfound= FALSE;

   for( i = 1; i <= LastItem; i++ ) {
      if( locationOfItem[i] == currentLocation ) {
         sprintf( buffer, getss, Item[i] );
	 get();
	 itemsfound = TRUE;
      }
   }
   if( !itemsfound )
      printf( s, nothingtoget ); 
}


void help() {                                                        // Print help screen
   printf( s, helpString );
}


void inventory() {                                                   // List of items held
   number i, found;
   found = FALSE;

   printf( s, youarecarrying );
   for( i = 0; i <= LastItem; i++ ) {
      if( Inventory[i] != NOTHING ) {
         printf( spsCr, Item[Inventory[i]] );
         found = TRUE;
      }
   }

   if( !found )
      printf( nothing );
}


void light() {                                                       // Light (matches or candle)
   bufferToLower();
   if( strlen( buffer ) < 7 ) {
      printf( s, badParse );
      return;
   }
   turnsPlayed++;
   if( strstr(buffer, candle) ) {
      if( (carryingItem(candle)  || itemIsHere(candle))    &&
          (carryingItem(matches) || itemIsHere(matches) )
        ) {
         turnCandle = turnsPlayed;
         printf( s, lightcandle );
      } else {
         if( ! (carryingItem(candle)  || itemIsHere(candle)) )
            printf( seeno, candle );
         if( ! (carryingItem(matches) || itemIsHere(matches)) )
            printf( seeno, matches );
      }
      return;
   }
   else if( strstr(buffer, match) ) {
      if( carryingItem(matches) || itemIsHere(matches) ) {
         printf( s, lightmatch );
      } else {
         printf( seeno, matches );
      }
      return;
   }
   printf( s, nopoint );
}


void look() {                                                        // Print description of surroundings
   printLocation( DETAILED );
   showItemsHere( DETAILED );
   showWhereYouCanGo();
}


void move() {
   timeElapsed++;
   turnsPlayed++;

   bufferToLower();
   if( (currentLocation == House3017Porch) && matNotMoved() && (strstr(buffer, mat)) ) {
       matMoved();
   }
   else if( (currentLocation == Mountain2) && (strstr(buffer, rock)) ) {
       printf( cannotmove, rock );
       return;
   }
   else if( (currentLocation == JinxCentre) && (strstr(buffer, statue)) ) {
       printf( cannotmove, statue );
       printf( s, groundshakes );
       if( !flooded() ) {
          printf( sdotCr, rainbegins );
          flood();
       }
       else {
          printf( dotCr );
       }
       return;
   }
   else
      printf( s, nopoint );
}


void travel() {                                                      // Travel
   Direction_t d;
   number len = strlen( buffer );

   if( buffer[0] == 'u' ) {
      d = Up;
   } else if( buffer[0] == 'd' ) {
      d = Down;
   } else if( buffer[0] == 'w' ) {
      d = West;
   } else if( buffer[0] == 'e' ) {
      d = East;
   } else if( buffer[0] == 'n' ) {
      if( len == 1 ) {
         d = North;
      } else {
         if( buffer[1] == 'w' ) {
            d = Northwest;
         } else {
            d = Northeast;
         }
      }
   } else if( buffer[0] == 's' ) {
      if( len == 1 ) {
         d = South;
      } else {
         if( buffer[1] == 'w' ) {
            d = Southwest;
         } else {
            d = Southeast;
         }
      }
   }

   if( Move[currentLocation][d] == UNAVAILABLE ) {
      if( (currentLocation == House3017Porch) && (d == North) ) {
         if( haveKey() ) {
            printf( s, usingkey );
            unlock3017FrontDoor();
         } else {   
            printf( s, doorlocked );
            return;
         }
      }
      else if( (currentLocation == House3017BackYard) && (d == North) ) {
         if( haveKey() ) {
            visited[134] = FALSE;
            printf( s, usingkey );
            unlock3017Shelter();
         } else {   
            printf( s, doorlocked );
            return;
         }
      }
      else if( (currentLocation == House4735Garage) && (d == Down) ) {
         visited[140] = FALSE;
         printf( s, openhatch );
         open4735Hatch();
      }
      else if( (currentLocation == ForestClearing) && (d == Down) ) {
         visited[142] = FALSE;
         printf( s, opengrate );
         openForestGrate();
      }
      else if( (currentLocation == Below4735Garage) && (d == Up) ) {
         printf( s, closedhatch );
         return;
      }
      else if( (currentLocation == BelowGrate) && (d == Up) ) {
         printf( s, closedgrate );
         return;
      } else {   
         printf( youcannotgo, Direction[d] );
         return;
      }
   }

   currentLocation = Move[currentLocation][d];
   if( visited[currentLocation] == FALSE ) {
      printLocation( DETAILED );
      showItemsHere( NORMAL );
      visited[currentLocation] = TRUE;
   } else {
      printLocation( NORMAL );
   }

   if( (currentLocation == House3750Bedroom) && !goneInRoom() ) {
      enterRoom();
   }

   checkAreas();

   turnsPlayed++;
   if( Indoors( currentLocation ) )
      timeElapsed += 1;
   else
      timeElapsed += 3;
}


void open() {                                                        // Open door
   turnsPlayed++;
   if( currentLocation == House3017Porch ) {
      if( is3017FrontDoorUnlocked() ) {
         printf( s, alreadyopen );
         currentLocation = House3017EntryHall;
      } else {
         if( haveKey() ) {
            printf( s, usingkey );
            unlock3017FrontDoor();
            currentLocation = House3017EntryHall;
         } else {   
            printf( s, doorlocked );
         }
      }
      return;
   }
   else if( currentLocation == House3017Front ) {
      currentLocation = House3017Porch;
      buffer[0] = 'n';
      buffer[1] = '\0';
      travel();
   }
   else if( currentLocation == House3017BackYard ) {
      if( is3017ShelterUnlocked() ) {
         printf( s, alreadyopen );
         currentLocation = House3017Shelter;
      } else {
         if( haveKey() ) {
            visited[134] = FALSE;
            printf( s, usingkey );
            unlock3017Shelter();
            currentLocation = House3017Shelter;
         } else {   
            printf( s, doorlocked );
         }
      }
      return;
   }
   else if( currentLocation == House5843Front ) {
      buffer[0] = 'n';
      buffer[1] = '\0';
      travel();
      return;
   }
   else if( currentLocation == House4735Front ) {
      buffer[0] = 'e';
      buffer[1] = '\0';
      travel();
      return;
   }
   else if( currentLocation == House3750Front ) {
      buffer[0] = 's';
      buffer[1] = '\0';
      travel();
      return;
   }
   else if( currentLocation == House4735Garage ) {
      if( isHatchOpen() ) {
         printf( s, alreadyopen );
      } else {
         visited[140] = FALSE;
         printf( s, openhatch );
         open4735Hatch();
      }
      return;
   }
   else if( currentLocation == ForestClearing ) {
      if( isGrateOpen() ) {
         printf( s, alreadyopen );
      } else {
         visited[142] = FALSE;
         printf( s, opengrate );
         openForestGrate();
      }
      return;
   }
   else if( currentLocation == Below4735Garage ) {
      if( isHatchOpen() )
         printf( s, alreadyopen );
      else
         printf( s, closedhatch );
      return;
   }
   else if( currentLocation == BelowGrate ) {
      if( isGrateOpen() )
         printf( s, alreadyopen );
      else
         printf( s, closedgrate );
      return;
   }
   else if( isAbandonedBusiness() ) {
      printf( s, barred );
      return;
   } else {
      printf( nothingtoopen );
      return;
   }
}


void pray() {                                                        // Pray
   if( turnsPlayed > 10 )
      timeElapsed -= chance() % 3;

   if( (currentLocation == 114) &&
       (turnsPlayed == turnBook) &&                                  // Did we light the candle,
       (turnBook == turnBell + 1) &&                                 //     ring the bell and
       (turnBell == turnCandle + 1)                                  //     read the book?
     ) {
      eliminateWraiths();
      printf( s, voiceofGod );
   } 
}


void quit() {                                                        // Quit prompt
   printf( sureyouwanttoquit );
   fgets( buffer, sizeof(buffer)-1, stdin );
   if( tolower(buffer[0]) == 'y' ) {
      GameOver();
   }
}


void readWords() {                                                   // Read
   bufferToLower();
   if( strlen( buffer ) < 6 ) {
      printf( s, badParse );
      return;
   }
   turnsPlayed++;
   timeElapsed++;
   if( strstr(buffer, book) ) {
      if( carryingItem(book) || itemIsHere(book) ) {
         turnBook = turnsPlayed;
         printf( s, prayer );
      } else {
         printf( s, nothingfound );
      }
      return;
   }
   else if( (strstr(buffer, parchment)) ||
            (strstr(buffer, rock))      ||
            (strstr(buffer, notice))    ||
            (strstr(buffer, magazine))  ||
            (strstr(buffer, newspaper)) ||
            (strstr(buffer, brochure))
          ) {
      examine();
      return;
   }
   printf( s, nothingtoread );
}


void ringbell() {                                                    // Ring (bell)
   bufferToLower();
   if( strlen( buffer ) < 6 ) {
      printf( s, badParse );
      return;
   }
   turnsPlayed++;
   if( strstr(buffer, bell) ) {
      if( carryingItem(bell) || itemIsHere(bell) ) {
         turnBell = turnsPlayed;
         printf( quoted, ring );
      }
      else {
         printf( seeno, bell );
      }
   }
   else {
      printf( s, nopoint );
   }
}


void speak() {                                                       // Speak
   if( strlen( buffer ) > 4 )
      printf( quoted, buffer+4 );
   else
      printf( blahblah );
   turnsPlayed++;
}


void status() {                                                      // Status
   printf( sofaryouplayed, turnsPlayed );
   if( timeElapsed > 0 ) {
      printf( in );
      printElapsedTime();
   }
   printf( dot );
   printf( printpointsearned, pointsEarned(), maxPoints() );
}


void type() {                                                        // Type
   number r = chance();
   if( inComputerRoom() ) {
      if( strstr(buffer, xyzzy) ) {
         if( in16bitComputerRoom() )
            printf( screensaysnothinghappens );
         else if( in32bitComputerRoom() )
            printf( screensaystwiceasmuchhappens );
      } else {
         printf( frontpanelflickers );
         if( r < 30 )
            printf( dotCr );
         else if( (r > 30) && (r < 70) )
            printf( diskshakes );
         else
            printf( tapemoves );
      }
   } else {
     printf( noComputer );
   }
   turnsPlayed++;
}


void unlock() {                                                      // Unlock door
   if( currentLocation == House3017Porch ) { 
      if( is3017FrontDoorUnlocked() ) {
         printf( s, notlocked );
      }
      else {
         if( haveKey() )
            open();
         else
            printf( s, needkey );
      }
      return;
   }
   else if( currentLocation == House3017BackYard ) {
      if( is3017ShelterUnlocked() ) {
         printf( s, notlocked );
      }
      else {
         if( haveKey() )
            open();
         else
            printf( s, needkey );
      }
      return;
   }
   else if( currentLocation == House4735Garage ) {
      printf( s, notlocked );
      return;
   }
   else if( currentLocation == ForestClearing ) {
      printf( s, notlocked );
      return;
   }
   else if( currentLocation == Below4735Garage ) {
      printf( s, closedhatch );
      return;
   }
   else if( currentLocation == BelowGrate ) {
      printf( s, closedgrate );
      return;
   }
   else if( isAbandonedBusiness() ) {
      printf( s, barred );
      return;
   } else {
      printf( nothingtounlock );
      return;
   }
}


void prompt() {                                                      // Prompt for player input
   printf( inp );
   fgets( buffer, sizeof(buffer)-1, stdin );
   buffer[strlen(buffer)-1] = '\0';
}


void playerAction() {                                                // Parse command and do it
   number len = strlen( buffer );

   if( len == 0 ) {
      return;
   }
   else if( len == 1 ) {
      buffer[0] = tolower( buffer[0] );
      if( buffer[0] == 'h' ) {                                       // help
         help();
      }
      else if( buffer[0] == 'i' ) {                                  // inventory
         inventory();
      }
      else if( buffer[0] == 'l' ) {                                  // look
         look();
      }
      else if( (buffer[0] == 'n') ||                                 // travel
               (buffer[0] == 's') ||
               (buffer[0] == 'e') ||
               (buffer[0] == 'w') ||
               (buffer[0] == 'u') ||
               (buffer[0] == 'd') )
      {
         travel();
      }
      else {
         printf( s, badParse );
      }
   }
   else if( len == 2 ) {
      buffer[1] = tolower( buffer[1] );
      if( buffer[0] == 'n' ) {
         if( (buffer[1] == 'e') || (buffer[1] == 'w') ) {
            travel();
         }
         else {
            printf( s, badParse );
         }
      }
      else if( buffer[0] == 's' ) {
         if( (buffer[1] == 'e') || (buffer[1] == 'w') ) {
            travel();
         }
         else {
            printf( s, badParse );
         }
      }
      else if( (buffer[0] == 'u') && (buffer[1] == 'p') ) {
            travel();
      }
      else {
         printf( s, badParse );
      }
   }
   else if( !strncasecmp(buffer, "go up", 5) ) {
      buffer[0] = 'u';
      buffer[1] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "down", 4)) || (!strncasecmp(buffer, "go down", 7)) ) {
      buffer[0] = 'd';
      buffer[1] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "northeast", 9)) || (!strncasecmp(buffer, "go northeast", 12)) ) {
      buffer[0] = 'n';
      buffer[1] = 'e';
      buffer[2] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "northwest", 9)) || (!strncasecmp(buffer, "go northwest", 12)) ) {
      buffer[0] = 'n';
      buffer[1] = 'w';
      buffer[2] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "southeast", 9)) || (!strncasecmp(buffer, "go southeast", 12)) ) {
      buffer[0] = 's';
      buffer[1] = 'e';
      buffer[2] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "southwest", 9)) || (!strncasecmp(buffer, "go southwest", 12)) ) {
      buffer[0] = 's';
      buffer[1] = 'w';
      buffer[2] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "north", 5)) || (!strncasecmp(buffer, "go north", 8)) ) {
      buffer[0] = 'n';
      buffer[1] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "south", 5)) || (!strncasecmp(buffer, "go south", 8)) ) {
      buffer[0] = 's';
      buffer[1] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "east", 4)) || (!strncasecmp(buffer, "go east", 7)) ) {
      buffer[0] = 'e';
      buffer[1] = '\0';
      travel();
   }
   else if( (!strncasecmp(buffer, "west", 4)) || (!strncasecmp(buffer, "go west", 7)) ) {
      buffer[0] = 'w';
      buffer[1] = '\0';
      travel();
   }
   else if( !strncasecmp(buffer, "drink", 5) ) {                     // drink
      drink();
   }
   else if( !strncasecmp(buffer, "drop every", 10) ) {               // drop everything
      dropall();
   }
   else if( !strncasecmp(buffer, "drop all", 8) ) {                  // drop all
      dropall();
   }
   else if( !strncasecmp(buffer, "drop", 4) ) {                      // drop
      drop();
   }
   else if( !strncasecmp(buffer, "eat", 3) ) {                       // eat
      eat();
   }
   else if( !strncasecmp(buffer, "enter", 5) ) {                     // enter
      open();
   }
   else if( !strncasecmp(buffer, "examine", 7) ) {                   // examine
      examine();
   }
   else if( !strncasecmp(buffer, "get every", 9) ) {                 // get everything
      getall();
   }
   else if( !strncasecmp(buffer, "get all", 7) ) {                   // get all
      getall();
   }
   else if( !strncasecmp(buffer, "get", 3) ) {                       // get
      get();
   }
   else if( !strncasecmp(buffer, "help", 4) ) {                      // help
      help();
   }
   else if( !strncasecmp(buffer, "inv", 3) ) {                       // inventory
      inventory();
   }
   else if( !strncasecmp(buffer, "kill", 4) ) {                      // kill
      attack();
   }
   else if( !strncasecmp(buffer, "light", 5) ) {                     // light
      light();
   }
#ifdef ENABLESAVELOAD
   else if( !strncasecmp(buffer, "load", 4) ) {                      // load
      loadGameState();
      status();
      look();
   }
#endif
   else if( !strncasecmp(buffer, "look", 4) ) {                      // look
      look();
   }
   else if( !strncasecmp(buffer, "move", 4) ) {                      // move
      move();
   }
   else if( !strncasecmp(buffer, "open", 4) ) {                      // open
      open();
   }
   else if( !strncasecmp(buffer, "pray", 4) ) {                      // pray
      pray();
   }
   else if( !strncasecmp(buffer, "quit", 4) ) {                      // quit
      quit();
   }
   else if( !strncasecmp(buffer, "read", 4) ) {                      // read
      readWords();
   }
   else if( !strncasecmp(buffer, "ring", 4) ) {                      // ring
      ringbell();
   }
#ifdef ENABLESAVELOAD
   else if( !strncasecmp(buffer, "save", 4) ) {                      // save
      saveGameState();
   }
#endif
   else if( !strncasecmp(buffer, "say", 3) ) {                       // say
      speak();
   }
   else if( !strncasecmp(buffer, "score", 5) ) {                     // score
      status();
   }
   else if( !strncasecmp(buffer, "take every", 10) ) {               // take everything (get everything)
      getall();
   }
   else if( !strncasecmp(buffer, "take all", 8) ) {                  // take all (get all)
      getall();
   }
   else if( !strncasecmp(buffer, "take", 4) ) {                      // take (get)
      get();
   }
   else if( !strncasecmp(buffer, "type", 4) ) {                      // type
      type();
   }
   else if( !strncasecmp(buffer, "unlock", 6) ) {                    // unlock
      unlock();
   }
   else {
      printf( s, badParse );
   }
}


void initialize() {
   WorldState = 1;
   turnsPlayed = 0;
   timeElapsed = 0;
   turnCandle = 0;
   turnBell = 0;
   turnBook = 0;
   wraithEncounters = 0;
   wraithStrength = INITIALWRAITHSTRENGTH;
   wraithAgitation = INITIALWRAITHAGITATION;
   turnsToAgitate = DEFAULTAGITATEFREQUENCY;

   Move[30][North] = 0;                                              // 3017 Front Door
   Move[37][North]  = 0;                                             // 3017 Shelter
   Move[38][South] = 0;
   Move[78][Down] = 0;                                               // 4735 Garage
   Move[140][Up] = 0;
   Move[121][Down] = 0;                                              // Forest Grate
   Move[142][Up] = 0;

   memset( buffer, 0, SCREENWIDTH );
   memset( visited, 0, sizeof(visited[0])*(LastLocation+1) );
   memset( Inventory, 0, sizeof(Inventory[0])*(LastItem+1) );

#ifdef ENABLESAVELOAD
   memset( &saveState, 0, sizeof(struct gameSaveState) );
#endif

   initLocations();
   initItems();
}


// ============================ Game Save / Load ===============================


#ifdef ENABLESAVELOAD

void loadGameState() {                                     // Load Game State
   int i;
   FILE *loadFile;

   loadFile = fopen( "jinx.sav", "rb" );

   if( loadFile == NULL ) {
      printf( "\nError opening jinx.sav file.  Perhaps one was never created.\n\n" );   
      return;             
   } else {
      fread( &saveState, sizeof(struct gameSaveState), 1, loadFile ); 
      if( ferror ( loadFile ) ) {
         printf( "\nError reading jinx.sav file.\n\n" );
      } else {

         WorldState       = saveState.WorldState;
         currentLocation  = saveState.currentLocation;
         turnsPlayed      = saveState.turnsPlayed;
         turnFlood        = saveState.turnFlood;
         turnCandle       = saveState.turnCandle;
         turnBell         = saveState.turnBell;
         turnBook         = saveState.turnBook;
         timeElapsed      = saveState.timeElapsed;
         wraithEncounters = saveState.wraithEncounters;
         wraithStrength   = saveState.wraithStrength;
         wraithAgitation  = saveState.wraithAgitation;
         turnsToAgitate   = saveState.turnsToAgitate;
         Move[30][North]  = saveState.Move30N;
         Move[37][North]  = saveState.Move37N;
         Move[38][South]  = saveState.Move38S;
         Move[78][Down]   = saveState.Move78D;
         Move[140][Up]    = saveState.Move140U;
         Move[121][Down]  = saveState.Move121D;
         Move[142][Up]    = saveState.Move142U;

         for( i=1; i<=LastLocation; i++ ) { 
            visited[i]    = saveState.visited[i];
         }

         for( i=1; i<=LastItem; i++ ) {
            Inventory[i]  = saveState.Inventory[i];
         }

         for( i=1; i<=LastItem; i++ ) {
            locationOfItem[i] = saveState.locationOfItem[i];
         }

         if( flooded() ) {                                 // If saved game had flooded
            for( i = 29; i < 38; i++ ) {                   // then remove house 3017
               Location[i] = muddyarea;
               Descript[i] = NULL;
            }
            Descript[31] = scattereddebris;                // and litter with debris
         }

         printf( "\nPrevious game restored.\n" );
      }

      fclose( loadFile );
   }

   return;
}


void saveGameState() {                                     // Save Game State
   int i;
   FILE *saveFile;

   saveFile = fopen( "jinx.sav", "wb+" );

   if( saveFile == NULL ) {
      printf( "\nError creating jinx.sav file.\n\n" );   
      return;             
   } else {

      saveState.WorldState       = WorldState;
      saveState.currentLocation  = currentLocation;
      saveState.turnsPlayed      = turnsPlayed;
      saveState.turnFlood        = turnFlood;
      saveState.turnCandle       = turnCandle;
      saveState.turnBell         = turnBell;
      saveState.turnBook         = turnBook;
      saveState.timeElapsed      = timeElapsed;
      saveState.wraithEncounters = wraithEncounters;
      saveState.wraithStrength   = wraithStrength;
      saveState.wraithAgitation  = wraithAgitation;
      saveState.turnsToAgitate   = turnsToAgitate;
      saveState.Move30N          = Move[30][North];
      saveState.Move37N          = Move[37][North];
      saveState.Move38S          = Move[38][South];
      saveState.Move78D          = Move[78][Down];
      saveState.Move140U         = Move[140][Up];
      saveState.Move121D         = Move[121][Down];
      saveState.Move142U         = Move[142][Up];

      for( i=1; i<=LastLocation; i++ ) { 
         saveState.visited[i]    = visited[i];
      }

      for( i=1; i<=LastItem; i++ ) {
         saveState.Inventory[i]  = Inventory[i];
      }

      for( i=1; i<=LastItem; i++ ) {
         saveState.locationOfItem[i] = locationOfItem[i];
      }

      fwrite( &saveState, sizeof(struct gameSaveState), 1, saveFile );
      if( ferror ( saveFile ) ) {
         printf( "\nError writing jinx.sav file.\n\n" );
      } else {
         printf( "\nSaved.\n" );
      }

      fclose( saveFile );
   }

   return;
}

#endif


// ================================= main() ====================================


int main() {
   initialize();
   clearScreen();
   printf( s, introText );
   look();

   while( running() ) {
      prompt();
      playerAction();
      checkFate();
   }

   return( NORMAL );
}

